#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

from Cloud import *

class CloudDNS(Cloud):
    """
    A class that handles actions to Cloud DNS via public methods listed.
    """
    
    service = "DNS"
    
    #Limits{{{

    def getLimits(self, type=None):
        """
        Lists all current api limits for CloudDB.  Passing type=list_types will show all available types of limits and passing
        type=<type> will show you assigned limits for that type.
        
        Arguments:
        type -- Limit type(string)(Optional)
        
        Returns:
        JSON results as a python dict
        """
        if type is not None:
            if re.search("list_types", str(type), re.IGNORECASE):
                self.baseURL('limits/types')
            else:
                self.baseURL('limits/'+str(type).lower())
        else:
            self.baseURL('limits')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    #}}}
    #Domains{{{

    def getDomains(self, name=None):
        """
        Lists all account domains.  Filter by name, if specified.  Use getDomainsId for more
        detailed response.
        
        Arguments:
        name -- Domain name(string)
        
        Returns:
        JSON results as a python dict
        """
        self.baseURL('domains')
        self.queryVar('name',name)
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getDomain(self, domainId, attrDict={}): 
        """
        List details for a specific domain, using the showRecords and showSubdomains 
        parameters that specify whether to request information for records and subdomains.
        
        Arguments:
        domainId --
        attrDict --
            showRecords -- (Optional)
            showSubdomains -- (Optional)
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('domains/'+str(domainId))
        self.attrDict = attrDict
        self.queryDict()
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getDomainChanges(self, domainId, changeSince):
        """
        Show all changes to the specified domain since the specified date/time in ISO format(2011-09-13T00:00:00-0500)
        
        Arguments:
        domainId --
        changeSince -- format 2011-05-19T08:07:08-0500 etc
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('domains/'+str(domainId)+'/changes')
        self.queryVar('since',changeSince)
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getDomainExport(self, domainId):
        """
        Export details of the specified domain id.
        
        Arguments:
        domainId --
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('domains/'+str(domainId)+'/export')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createDomains(self, attrList, returnResults=False):
        """
        Create domain requests given the provided info.  The recordsList is a list of "records" using
        name, type(A,CNAME, etc), data and ttl. The subdomains is a list of "domains" using name, comment 
        and emailAddress.  See api docs for examples.
        
        Arguments:
        attrList: python list of dictionaries: 
            name -- (Required)
            emailAddress -- (Required)
            ttl -- (Optional)
            commment -- (Optional)
            #recordsList -- (Optional)
            #subdomains -- (Optional)
        returnResults -- True / False - receive the Async Json (True) or the Response Code (Optional)

        Returns: Depends on returnResults arg.
        """
        self.baseURL('domains')
        self.postKey = 'domains'
        self.attrList = attrList
        self.postKeyListDict()
        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=returnResults)    
        
    def createDomain(self, name, email, attrDict={}, returnResults=False):
        """
        Create domain request given the provided info.  The recordsList is a list of "records" using
        name, type(A,CNAME, etc), data and ttl. The subdomains is a list of "domains" using name, comment 
        and emailAddress.  See api docs for examples.
        
        Arguments:
        name -- (Required)
        email -- (Required)
        attrDict: python dictionary:
            ttl -- (Optional)
            commment -- (Optional)
            #recordsList -- (Optional)
            #subdomains -- (Optional)
        returnResults -- True / False - receive the Async Json (True) or the Response Code (Optional)
        
        Returns: Depends on returnResults arg.
        """
        attrDict['name'] = name
        attrDict['emailAddress'] = email
        attrList = [attrDict]
        return self.createDomains(attrList, returnResults)
        
    def importDomain(self, contents, contentType="BIND_9"):
        """
        Import a new domain with the configuration specified by the request.  Currently "BIND_9" is the 
        only type supported and is defaulted to.
        
        Arguments:
        contentType --
        contents --
        
        Returns: 
        JSON results as a python dict
        """
        self.baseURL('domains/import')
        self.postKey = 'domains' 
        attrList = [{'contentType':contentType,'contents':contents}] 
        self.attrList = attrList
        self.postKeyListDict()
        return apiRequest(self, self.endpoint, "POST", self.data, returnResults=True)
        
    def updateDomains(self, domainList):
        """
        Update the configuration of a domain or domains.  This allows for updating of comments, ttl's or
        emailAddresses set on the domain.  Domain dict should include id and optionally comment, ttl and 
        emailAddress
        
        Arguments:
        domainList --
            id --
            emailAddress -- (Optional)
            ttl -- (Optional)
            comment -- (Optional)

        Returns:
        JSON results as a python dict
        """
        try:
            if len(domainList) > 1: #updating multiple
                self.baseURL('domains')
                self.postKey = 'domains' 
                self.postKeyListDict()
            else: #updating just 1
                self.baseURL('domains/'+str(domainList[0]['id']))
                del domainList[0]['id']
                self.attrDict = domainList[0]
                self.postDict()
            return apiRequest(self, self.endpoint, "PUT", self.postData, returnResults=True)
        except:
            raise RSException("updateDomains")
            
    def updateDomain(self, domainId, domainDict={}):
        """
        Update the configuration of a domain.  This allows for updating of comments, ttl's or
        emailAddresses set on the domain.  Domain dict should include id and optionally comment, ttl and 
        emailAddress
        
        Arguments:
        domainId --
        domainList --
            email -- (Optional)
            ttl -- (Optional)
            comment -- (Optional)

        Returns:
        JSON results as a python dict
        """
        domainDict['id'] = domainId
        domainList = [domainDict]
        return self.updateDomains(domainList)
            
    def deleteDomain(self, domainId, deleteSubdomains=False, returnResults=False):
        """
        Remove domain from account.  Pass True to delete subdomains.
        
        Arguments:
        domainList -- list of domain ids to delete
        deleteSubdomains -- (Optional)
        returnResults -- True / False - receive the Async Json (True) or the Response Code (Optional)
        
        Returns: Depends on returnResults arg.
        """
        return self.deleteDomains([domainId],deleteSubdomains,returnResults)
        
    def deleteDomains(self, domainList, deleteSubdomains=False, returnResults=False):
        """
        Remove domain(s) from account.  Pass True to delete subdomains.
        
        Arguments:
        domainList -- list of domain ids to delete
        deleteSubdomains -- (Optional)
        returnResults -- True / False - receive the Async Json (True) or the Response Code (Optional)
        
        Returns: Depends on returnResults arg.
        """

        try:
            self.baseURL('domains')
            self.queryVar('id',str(domainList[0]))
            if len(domainList) > 1:
                for i in range(1,len(domainList)):
                    self.queryVar('id',str(domainList[i]))
            if deleteSubdomains:
                self.queryVar('deleteSubdomains','true')
            return apiRequest(self, self.endpoint, "DELETE", returnResults=returnResults)
        except:
            raise RSException("deleteDomain")

    #}}}
    #Subdomains{{{

    def getSubdomains(self, domainId):
        """
        List domains that are subdomains of the specified domain.
        
        Arguments:
        domainId --
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('domains/'+str(domainId)+'/subdomains')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    #}}}
    #Records{{{

    def getRecord(self, domainId, recordId=None):
        """
        List all records configured for the domain. SOA cannot be modified. Pass recordId to get 
        record details.
        
        Arguments:
        domainId --
        recordId --
        """
        if recordId is not None:
            self.baseURL('domains/'+str(domainId)+'/records/'+str(recordId))
        else:
            self.baseURL('domains/'+str(domainId)+'/records')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getRecords(self, domainId, recordType, attrDict={}): #name=None, data=None):
        """
        List all records for the specified domain of the specified type that match the specified 
        name and/or data.
        
        Arguments:
        domainId --
        recordType --
        attrDict: 
            name -- (Optional)
            data -- (Optional)
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('domains/'+str(domainId)+'/records')
        self.queryVar('type',recordType)
        self.attrDict = attrDict
        self.queryDict()
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createRecords(self, domainId, recordList,returnResults=False):
        """
        Add new record(s) to the specified domains in a "records" list with name, type and data required.
        
        Arguments:
        domainId -- Domain Id
        recordList -- Python List of Dictionaries:
            type -- Specifies the record type to add.
            name -- Specifies the name for the domain or subdomain. Must be a valid domain name.
            data -- The data field for PTR, A, and AAAA records must be a valid IPv4 or IPv6 IP address
            priority -- (MX and SRV records only) Must be an integer from 0 to 65535.
            ttl -- Must be greater than 300. Defaults to the domain TTL or 3600 (Optional)
            comment -- If included, its length must be less than or equal to 160 characters. (Optional)
        returnResults -- True / False - receive the Async Json (True) or the Response Code (Optional)
        
        Returns: Depends on returnResults arg.
        """
        self.baseURL('domains/'+str(domainId)+'/records')
        self.attrList = recordList
        self.postKey = 'records'
        self.postKeyListDict()
        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=returnResults)
        
    def createRecord(self, domainId, type, name, data, recordDict={},returnResults=False):
        """
        Add new record to the specified domains in a "records" list with name, type and data required.
        
        Arguments:
        domainId -- Domain Id
        type -- Specifies the record type to add.
        name -- Specifies the name for the domain or subdomain. Must be a valid domain name.
        data -- The data field for PTR, A, and AAAA records must be a valid IPv4 or IPv6 IP address
        recordDict --
            priority -- (MX and SRV records only) Must be an integer from 0 to 65535.
            ttl -- Must be greater than 300. Defaults to the domain TTL or 3600 (Optional)
            comment -- If included, its length must be less than or equal to 160 characters. (Optional)
        returnResults -- True / False - receive the Async Json (True) or the Response Code (Optional)
        
        Returns: Depends on returnResults arg.
        """
        recordDict['type'] = type
        recordDict['name'] = name
        recordDict['data'] = data
        recordList = [recordDict]
        return self.createRecords(domainId,recordList,returnResults)
        
    def updateRecords(self, domainId, recordList):
        """
        Modify the configuration of records in the domain.
        
        Arguments:
        domainId -- Domain Id
        recordList -- 
            rId -- Record Id
            type -- Specifies the record type to add.
            name -- Specifies the name for the domain or subdomain. Must be a valid domain name.
            data -- The data field for PTR, A, and AAAA records must be a valid IPv4 or IPv6 IP address
            priority -- (MX and SRV records only) Must be an integer from 0 to 65535.
            ttl -- Must be greater than 300. Defaults to the domain TTL or 3600 (Optional)
            comment -- If included, its length must be less than or equal to 160 characters. (Optional)
        
        Returns:
        JSON results as python dict
        """
        
        try:
            if len(recordList) > 1:
                self.baseURL('domains/'+str(domainId)+'/records')
                self.postKey = 'records'
                self.attrList = recordList
                self.postKeyListDict()
            else:
                recordDict = recordList[0]
                self.baseURL('domains/'+str(domainId)+'/records/'+str(recordDict['id']))
                del recordDict['id']
                self.attrDict = recordDict
                self.postDict()
            return apiRequest(self, self.endpoint, "PUT", self.postData, returnResults=True)
        except:
            raise RSException("updateRecord")
        
    def updateRecord(self, domainId, rId, recordDict):
        """
        Modify the configuration of records in the domain.
        
        Arguments:
        domainId -- Domain Id
        rId --  Record Id
        recordDict -- 
            type -- Specifies the record type to add.
            name -- Specifies the name for the domain or subdomain. Must be a valid domain name.
            data -- The data field for PTR, A, and AAAA records must be a valid IPv4 or IPv6 IP address
            priority -- (MX and SRV records only) Must be an integer from 0 to 65535.
            ttl -- Must be greater than 300. Defaults to the domain TTL or 3600 (Optional)
            comment -- If included, its length must be less than or equal to 160 characters. (Optional)
        
        Returns:
        JSON results as python dict
        """
        recordDict['id'] = rId
        #recordDict['type'] = type
        #recordDict['name'] = name
        recordList = [recordDict]
        return self.updateRecords(domainId,recordList) 
        
    def deleteRecords(self, domainId, recordIdList,returnResults=False):
        """
        Remove records from the domain.
        
        Arguments:
        domainId --
        recordIdList -- List containing record ids
        """
        try:
            if len(recordIdList) > 1:
                self.baseURL('domains/'+str(domainId)+'/records')
                for i in range(0,len(recordIdList)):
                    self.queryVar('id',recordIdList[i])
            else:
                self.baseURL('domains/'+str(domainId)+'/records/'+str(recordIdList[0]))
            return apiRequest(self, self.endpoint, "DELETE", returnResults=returnResults)
        except:
            raise RSException("deleteRecord")
        
    def deleteRecord(self, domainId, recordId, returnResults=False):
        """
        Remove a record from the domain.
        
        Arguments:
        domainId -- Domain Id
        recordId -- Record Id
        """
        recordIdList = [recordId]
        return self.deleteRecords(domainId,recordIdList,returnResults) 

    #}}}
    #Rdns{{{

    def getRdns(self, serviceType, deviceUri, recordId=None):
        """
        List all PTR records configured for a Rackspace Cloud device or details on a specified record.
        
        Arguments:
        serviceType -- Current available options are cloudServersOpenStack or cloudLoadBalancers
        deviceUri -- Device href for associated service
        recordId -- Id specified in ptr record listing
        
        Returns:
        JSON results as python dict
        """
        if recordId is None:
            endpoint = self.publicURL + "/rdns/%s?href=%s" % (str(serviceType), str(deviceUri))
        else:
            endpoint = self.publicURL + "/rdns/%s/%s?href=%s" % (str(serviceType), str(recordId), str(deviceUri))
        return apiRequest(self, endpoint, "GET", returnResults=True)
        
    def createRdns(self, recordList, serviceType, deviceUri):
        """
        Add new PTR record(s) for the specified Cloud device.
        
        Arguments:
        recordList -- { 'records': [ { 'name': required, 'type': 'PTR', 'data': required, 'ttl': optional } ] } etc
        serviceType -- cloudServersOpenStack or cloudLoadBalancers
        deviceUri -- Uri from service
        
        Returns:
        JSON result as python dict
        """
        data = { }
        endpoint = self.publicURL + "/rdns"
        data['recordsList'] = recordList
        data['link'] = { 'content': '', 'href':str(deviceUri), 'rel': str(deviceUri) }
        return apiRequest(self, endpoint, "POST", data, returnResults=True)
        
    def updateRdns(self, recordList, serviceType, deviceUri):
        """
        Modify existing PTR record(s) for specified Cloud device(s).
        
        Arguments:
        recordList -- { 'records': [ { 'name': required, 'id': required, 'type': 'PTR', 'data': required, 'ttl': optional } ] } etc
        serviceType -- cloudServersOpenStack or cloudLoadBalancers
        deviceUri -- Uri from service
        
        Returns:
        JSON result as python dict
        """
        #self.baseURL('rdns')
        data = { }
        endpoint = self.publicURL + "/rdns"
        data['recordsList'] = recordList
        data['link'] = { 'content': '', 'href':str(deviceUri), 'rel': str(deviceUri) }
        return apiRequest(self, self.endpoint, "PUT", data, returnResults=True)
        
    def deleteRdns(self, serviceType, deviceUri, ip=None):
        """
        Remove one or all PTR records associated with a Rackspace Cloud device. Use the optional ip query parameter to 
        specify a specific record to delete. Omitting this parameter removes all PTR records associated with the 
        specified device.
        
        Arguments:
        serviceType -- cloudServersOpenStack or cloudLoadBalancers
        deviceUri -- Uri from service
        ip -- Optional ipv4 or ipv6 addy
        
        Returns:
        status code
        """
        endpoint = self.publicURL + "/rdns/%s?href=%s" % (str(serviceType), str(deviceUri))
        if ip is not None:
            endpoint = endpoint + "&ip=%s" % str(ip)
        return apiRequest(self, endpoint, "DELETE")

    #}}}

    def getAsync(self, url, detail=False):
        """
        List status of the specified asynchronous request. Display details, as specified by the showDetails parameter.
        
        Arguments:
        url -- callbackUrl(string)
        detail -- (bool)
        
        Returns:
        JSON results as python dict
        """
        self.baseURL(url)
        self.queryVar('showDetails',str(detail))
        self.queryVar('showDetails',str('true'))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

#eof



#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

from Cloud import *

class CloudLB(Cloud):
    
    service = "LoadBalancers"
   
    #Limits {{{

    def getLimits(self, absolute=False):
        """
        Return the current rate limits for the account.  Pass True to return absolute limits.
        
        Arguments:
        absolute -- Bool, set True for absolute limits.  Defaults to False
        
        Returns: JSON results as python dict
        """
        if absolute:
            self.baseURL('loadbalancers/absolutelimits.json') 
        else:
            self.baseURL('loadbalancers/limits.json') 

        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getAbsoluteLimits(self):
        """
        Return the current absolute limits for the account.  
        
        Arguments:
        absolute -- Bool, set True for absolute limits.  Defaults to False
        
        Returns: JSON results as python dict
        """
        return self.getLimits(True)

    # }}}
    #Load Balancers {{{

    def getLBs(self):
        """
        List all load balancers configured for the account (IDs, names and status only).  
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getLB(self, lbId):
        """
        Retrieve load balancer details.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createLB(self, name, port, nodes, protocol, virtualIps, attrDict={}):
        """
        Create a new load balancer with the configuration defined by the request.
        
        Arguments:
        name -- 
        port -- 
        nodes -- A list containing a dictionary for each with all of the following key/values:
            address -- IP address of node (Required)
            port -- Port number for the service you are load balancing on the node. (Required)
            condition -- ENABLED / DISABLED (Required)
            id -- Id of node (Optional)
            status -- ONLINE / OFFLINE (Optional)
        protocol -- HTTP / HTTPS / DNS / IMAPS / LDAP / MYSQL / POP3 / SMTP / TCP / UDP / SFTP etc.
        virtualIps -- A list containing a dictionary with any of the following key/values:
            type -- PUBLIC / PRIVATE (Required)
            address -- IP address of virtual IP (Optional)
            id -- Id of existing LB virtual ip. (Optional)
            ipVersion -- IPV4 / IPV6 (Optional)

        attrDict Options:
            accessList -- Network access controls to be applied to the load balancer's virtual IP address.
            algorithm -- Algorithm that defines how traffic should be directed between back-end nodes.
            connectionLogging -- Current connection logging configuration.
            connectionThrottle -- Limits on the number of connections per IP address
            healthMonitor -- The type of health monitor check to perform.
            metadata -- Information that can be associated with each load balancer for the client's personal use.
            port -- Port number for the service you are load balancing on the node.
            timeout --
            sessionPersistence -- Specifies whether multiple requests from clients are directed to the same node.

        Returns: status code
        """

        self.attrDict = attrDict
        self.postKey = 'loadBalancer'
        self.baseURL('loadbalancers')

        #req args
        self.postKeyVar('name',name)
        self.postKeyVar('nodes',nodes)
        self.postKeyVar('port',port)
        self.postKeyVar('protocol',protocol)
        self.postKeyVar('virtualIps',virtualIps)

        #optional attrDict args
        self.postKeyDict()

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def updateLB(self, lbId, attrDict={}):
        """
        Updates the attributes of the specified load balancer.
        
        Arguments:
        lbId -- Load Balancer id

        attrDict Options:
            name -- Name of the load balancer to create
            protocol -- Protocol of the service which is being load balanced.
            algorithm -- Algorithm that defines how traffic should be directed between back-end nodes.
            port -- Port number for the service you are load balancing.
            connectionLogging -- true/false

        Returns: status code
        """

        self.attrDict = attrDict
        self.postKey = 'loadBalancer'
        self.baseURL('loadbalancers/'+str(lbId))

        #optional attrDict args
        self.postKeyDict()

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteLB(self, lbId):
        """
        Removes the specified load balancer and its associated configuration from the account.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: status code
        """
        self.baseURL('loadbalancers/'+str(lbId))
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Error Pages {{{

    def getErrorPage(self, lbId):
        """
        List error page configured for the specified load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as Python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/errorpage')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def setErrorPage(self, lbId, content):
        """
        Set custom error page for the specified load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        contents -- HTML markup(string)
        
        Returns: response code
        """
        self.postKey = 'errorpage'
        self.baseURL('loadbalancers/'+str(lbId)+'/errorpage')
        self.postKeyVar('content',content)
        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteErrorPage(self, lbId):
        """
        Delete custom error page for the specified load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: status code
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/errorpage')
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}        
    #Stats {{{

    def getStats(self, lbId):
        """
        Provides detailed stats output of specified LB.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/stats.json')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Nodes {{{

    def getNode(self, lbId, nId):
        """
        List a specific node per a specified load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        nId -- Node id
        
        Returns: JSON results as python dict
        """

        self.baseURL('loadbalancers/'+str(lbId)+'/nodes/'+str(nId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getNodes(self, lbId):
        """
        List nodes per a specified load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        nId -- Node id
        
        Returns: JSON results as python dict
        """

        self.baseURL('loadbalancers/'+str(lbId)+'/nodes')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createNode(self, lbId, attrList=[]):
        """
        Add a node to a specified load balancer.
        
        Arguments:
        lbId -- load balancer id 

        attrList -- python list of dict options:
            address -- IP address or domain name for the node.(Required)
            condition -- Condition for the node.  ENABLED / DISABLED / DRAINING (Required)
            port -- Port number for the service you are load balancing.(Required)
            type -- Type of node to add: PRIMARY / SECONDARY (Optional)
            weight -- Weight of node to add. Must be an integer from 1 to 100.(Optional)
        
        Returns: status code
        """

        self.attrList = attrList
        self.postKey = 'nodes'
        self.baseURL('loadbalancers/'+str(lbId)+'/nodes')

        #attrDict args (required+optional)
        self.postKeyListDict()

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def updateNode(self, lbId, nId, attrDict):
        """
        Modify a node per a specified load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        nId -- Node id

        attrDict Options:
            condition -- Condition for the node.  ENABLED / DISABLED / DRAINING
            type -- Type of node to add: PRIMARY / SECONDARY
            weight -- Weight of node to add. Must be an integer from 1 to 100.
        
        Returns: status code
        """

        self.attrDict = attrDict
        self.postKey = 'node'
        self.baseURL('loadbalancers/'+str(lbId)+'/nodes/'+str(nId))
        
        #optional attrDict args (at least one req)
        self.postKeyDict()

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteNode(self, lbId, nodeList=[]):
        """
        Remove node(s) per a specified load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        nodeList -- Python list containing nodes to delete.
        
        Returns: status code
        """
        if len(nodeList) == 1:
            self.baseURL('loadbalancers/'+str(lbId)+'/nodes/'+str(nodeList[0]))
        if len(nodeList) > 1:
            self.baseURL('loadbalancers/'+str(lbId)+'/nodes')
            for node in nodeList:
                self.queryVar('id',str(node))
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Virtual IP's {{{

    def getVirtualIPs(self, lbId):
        """
        List all virtual IPs associated with a load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/virtualips/')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getVirtualIP(self, lbId, vpId):
        """
        List a specific virtual IP associated with a load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        vpId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/virtualips/'+str(vpId))
        endpoint = self.publicURL + "/loadbalancers/%s/virtualips/%s" % (str(lbId),str(vpId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Allowed Domains {{{

    def getAllowedDomains(self):
        """
        List all allowed domains.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/alloweddomains')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createVirtualIP(self, lbId, typeIp, ipVersion='IPV6'):
        """
        Create a new virtual IP (v.6) with the configuration defined by the request.
        
        Arguments:
        lbId -- Load Balancer id 
        typeIp -- PUBLIC/SERVICENET 
        ipVersion -- IPV6 
        
        Returns: status code
        """

        self.baseURL('loadbalancers/'+str(lbId)+'/virtualips')

        #required args
        self.postVar('type',typeIp)
        self.postVar('ipVersion',ipVersion)

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def deleteVirtualIP(self, lbId, vipList=[]):
        """
        Remove virtual ip(s) from the load balancer.
        
        Arguments:
        lbId -- Load Balancer id 
        vipList -- Python list containing virtual Ip's to delete. 
        
        Returns: status code
        """

        if len(vipList) == 1:
            self.baseURL('loadbalancers/'+str(lbId)+'/virtualips/'+str(vipList[0]))
        if len(vipList) > 1:
            self.baseURL('loadbalancers/'+str(lbId)+'/virtualips')
            for vip in vipList:
                self.queryVar('id',str(vip))

        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Usage {{{

    def getBillable(self, startTime, endTime, offset=None, limit=None):
        """
        List paginated billable load balancers.
        
        Arguments:
        startTime -- start date; format is YYYY-MM-DD 
        endTime -- end date; format is YYYY-MM-DD
        offset -- (Optional)
        limit -- (Optional)

        Returns: JSON results as python dict
        """

        self.baseURL('loadbalancers/billable')
        self.queryVar('startTime',startTime)
        self.queryVar('endTime',endTime)
        if offset:
            self.queryVar('offset',offset)
        if limit:
            self.queryVar('limit',limit)

        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getUsage(self, lbId=None, startTime=None, endTime=None, offset=None, limit=None):
        """
        List types of usage statistics (transfer activity, average number of
        connections, virtual IP count) over a date range or last 24 hours ("current").
        
        Arguments:
        lbId -- load balancer id (Optional)
        startTime -- start date; format is YYYY-MM-DD (Optional, if not set then current=true with lbId) 
        endTime -- end date; format is YYYY-MM-DD (Optional, if not set then current=true with lbId) 
        offset -- (Optional)
        limit -- (Optional)

        Returns: JSON results as python dict
        """

        if (startTime==None or endTime==None) and lbId is not None:
            self.baseURL('loadbalancers/'+str(lbId)+'/usage/current')
        else:
            self.baseURL('loadbalancers/usage')
            self.queryVar('startTime',startTime)
            self.queryVar('endTime',endTime)
        if offset:
            self.queryVar('offset',offset)
        if limit:
            self.queryVar('limit',limit)

        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Access Lists {{{

    def getAccessList(self, lbId):
        """
        List all access list associated with a load balancer.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/accesslist')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createAccessList(self, lbId, ipList=[]):
        """
        Create a new access list with the configuration defined by the request.
        
        Arguments:
        lbId -- Load Balancer id 
        ipList -- python list of dictionary key/value args:
            address -- ip address for item to add 
            alType -- ALLOW / DENY 

        Returns: status code
        """

        self.postKey = 'accessList'
        self.baseURL('loadbalancers/'+str(lbId)+'/accesslist') 
        self.attrList = ipList

        #required args
        self.postKeyListDict()

        return apiRequest(self, self.endpoint, "POST", self.postData)
        
    def updateAccessList(self, lbId, ipList=[]):
        """
        Update an access list with the configuration defined by the request.
        
        Arguments:
        lbId -- Load Balancer id 
        ipList -- python list of dictionaries with these required args:
            address -- ip address for item to add 
            type -- ALLOW / DENY 

        Returns: status code
        """
        return self.createAccessList(lbId,ipList)
        
    def deleteAccessList(self, lbId):
        """
        Remove access list from the load balancer.
        
        Arguments:
        lbId -- Load Balancer id 
        
        Returns: status code
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/accesslist')
        return apiRequest(self, self.endpoint, "DELETE")
        
    def deleteAccessListNetworkItems(self, lbId, ipList=[]):
        """
        Remove network item(s) from a load balancer's access list.
        
        Arguments:
        lbId -- Load Balancer id
        ipList -- Python list containing network item id's to delete. 
        
        Returns: status code
        """

        if len(ipList) == 1:
            self.baseURL('loadbalancers/'+str(lbId)+'/accesslist/'+str(ipList[0]))
        if len(ipList) > 1:
            self.baseURL('loadbalancers/'+str(lbId)+'/accesslist')
            for ipItm in ipList:
                self.queryVar('id',str(ipItm))

        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Monitor Health {{{

    def getMonitorHealth(self, lbId):
        """
        Retrieve health monitor configuration (if one exists). 
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/healthmonitor')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def deleteMonitorHealth(self, lbId):
        """
        Remove health monitor from the load balancer.
        
        Arguments:
        lbId -- Load Balancer id 
        
        Returns: status code
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/healthmonitor')
        return apiRequest(self, self.endpoint, "DELETE")
        
    def updateMonitorHealthConn(self, lbId, attempts, delay, timeout, type):
        """
        Update Health Monitor connections with the configuration defined by the request.
        
        Arguments:
        lbId -- Load Balancer id 
        attempts -- attempts before deactivation 
        delay --
        timeout -- 
        type -- 

        Returns: status code
        """

        self.baseURL('loadbalancers/'+str(lbId)+'/healthmonitor')

        #required args
        self.postVar('attemptsBeforeDeactivation',attempts)
        self.postVar('delay',delay)
        self.postVar('timeout',timeout)
        self.postVar('type',type)

        return apiRequest(self, self.endpoint, "PUT", self.postData)
    def updateMonitorHealthHttp(self, lbId, attempts, bRegex, delay, path, sRegex, timeout, type, hostHeader=None):
        """
        Update Health Monitor HTTP/HTTPS with the configuration defined by the request.
        
        Arguments:
        lbId -- Load Balancer id 
        attempts -- attempts before deactivation 
        bRegex -- bodyRegex
        delay -- 
        path -- 
        sRegex -- statusRegex
        timeout -- 
        type -- 
        hostHeader -- 

        Returns: status code
        """

        self.baseURL('loadbalancers/'+str(lbId)+'/healthmonitor')

        #required args
        self.postVar('attemptsBeforeDeactivation',attempts)
        self.postVar('bodyRegex',bRegex)
        self.postVar('delay',delay)
        self.postVar('path',path)
        self.postVar('statusRegex',sRegex)
        self.postVar('timeout',timeout)
        self.postVar('type',type)
        self.postVar('hostHeader',hostHeader)

        return apiRequest(self, self.endpoint, "PUT", self.postData)

    # }}}
    #Sessions {{{

    def getSession(self, lbId):
        """
        Retrieve session persistence configuration.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/sessionpersistence') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def updateSession(self, lbId, type):
        """
        Update session persistence with the configuration defined by the request.
        
        Arguments:
        lbId -- Load Balancer id 
        type -- persistence type - HTTP_COOKIE / SOURCE_IP 

        Returns: status code
        """

        self.postKey = 'sessionPersistence'
        self.baseURL('loadbalancers/'+str(lbId)+'/sessionpersistence') 

        #required args
        self.postKeyVar('persistenceType',type)

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteSession(self, lbId):
        """
        Disable session persistence from the load balancer.
        
        Arguments:
        lbId -- Load Balancer id 
        
        Returns: status code
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/sessionpersistence') 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Connections {{{

    def getConnection(self, lbId):
        """
        Retrieve connection logging configuration.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/connectionlogging') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def setConnection(self, lbId, enabled):
        """
        Enable/disable connection logging.
        
        Arguments:
        lbId -- Load Balancer id 
        enabled -- true / false  

        Returns: status code
        """

        self.postKey = 'connectionLogging'
        self.baseURL('loadbalancers/'+str(lbId)+'/connectionlogging') 

        #required vars
        self.postKeyVar('enabled',enabled)

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def getConnectionThrottle(self, lbId):
        """
        Retrieve connection throttle configuration.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/connectionthrottle') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def updateConnectionThrottle(self, lbId, attrDict={}):
        """
        Update connection throttle configuration defined by the request.
        
        Arguments:
        lbId -- Load Balancer id 
        attrDict -- python dict options:
            maxConnectionRate -- (Optional)
            maxConnections -- (Optional)
            minConnections -- (Optional)
            rateInterval -- (Optional)

        Returns: status code
        """

        self.baseURL('loadbalancers/'+str(lbId)+'/connectionthrottle') 
        self.attrDict = attrDict

        #attrDict optional args
        self.postDict()    

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteConnectionThrottle(self, lbId):
        """
        Remove connection throttle configuration from the load balancer.
        
        Arguments:
        lbId -- Load Balancer id (Required)
        
        Returns: status code
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/connectionthrottle') 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Content Caching {{{

    def getContentCaching(self, lbId):
        """
        Retrieve content caching configuration.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/contentcaching') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def setContentCaching(self, lbId, enabled):
        """
        Enable/disable content caching.
        
        Arguments:
        lbId -- Load Balancer id 
        enabled -- true / false 

        Returns: status code
        """

        self.baseURL('loadbalancers/'+str(lbId)+'/contentcaching') 
        self.postKey = 'contentCaching'

        #required args
        self.postKeyVar('enabled',enabled)

        return apiRequest(self, self.endpoint, "PUT", self.postData)

    # }}}
    #Protocols {{{

    def getProtocols(self):
        """
        List supported load balancing protocols.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/protocols') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Algorithms {{{

    def getAlgorithms(self):
        """
        List supported load balancing algorithms.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/algorithms') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #SSL Termination {{{

    def getSSLTermination(self, lbId):
        """
        List load balancers SSL termination configuration.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: JSON results as python dict
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/ssltermination') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def setSSLTermination(self, lbId, attrDict):
        """
        Update/configure SSL termination.
        
        Arguments:
        lbId -- Load Balancer id 

        attrDict -- python dictionary options:
            securePort --  (Required)
            privatekey -- (Required)
            certificate --  (Required)
            intermediatecertificate -- (Required for intermediate SSL Termination)
            enabled -- true / false  (Optional)
            secureTrafficOnly -- true / false (Optional)
        
        Returns: status code
        """

        self.postKey = 'sslTermination'
        self.baseURL('loadbalancers/'+str(lbId)+'/ssltermination') 
        self.attrDict = attrDict

        self.postKeyDict()

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteSSLTermination(self, lbId):
        """
        Remote SSL Termination.
        
        Arguments:
        lbId -- Load Balancer id
        
        Returns: status code
        """
        self.baseURL('loadbalancers/'+str(lbId)+'/ssltermination') 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Metadata {{{

    def getMetadata(self, lbId, metaId=None, nodeId=None):
        """
        Retrieve metadata associated with load balancer.
        
        Arguments:
        lbId -- Load Balancer id 
        metaId -- Meta id (Optional)
        nodeId -- Node id (Optional)
        
        Returns: JSON results as python dict
        """

        self.postKey = 'metadata'

        if nodeId is not None:
            req = 'loadbalancers/'+str(lbId)+'/nodes/'+str(nodeId)+'/metadata'
        else:
            req = 'loadbalancers/'+str(lbId)+'/metadata'

        if metaId is not None:
            self.baseURL(req+'/'+str(metaId))
        else:
            self.baseURL(req)

        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createMetadata(self, lbId, metaList=[], nodeId=None):
        """
        Create metadata per a load balancer.
        
        Arguments:
        lbId -- Load Balancer id 
        nodeId -- (Optional)
        metaList -- python list of meta key/value dictionaries:
            key -- 
            value --

        Returns: status code
        """

        self.postKey = 'metadata'
        self.attrList = metaList

        if nodeId is not None:
            self.baseURL('loadbalancers/'+str(lbId)+'/nodes/'+str(nodeId)+'/metadata')
        else:
            self.baseURL('loadbalancers/'+str(lbId)+'/metadata')

        #required metaList args
        self.postKeyListDict()

        return apiRequest(self, self.endpoint, "POST", self.postData)
        
    def updateMetadata(self, lbId, metaId, value, nodeId=None):
        """
        Update metadata per a load balancer (and optionally with a node) defined by the request.
        
        Arguments:
        lbId -- Load Balancer id 
        metaId -- Meta id 
        value -- 
        nodeId -- Node id (Optional)

        Returns: status code
        """

        self.postKey = 'meta'

        if nodeId is not None:
            self.baseURL('loadbalancers/'+str(lbId)+'/nodes/'+str(nodeId)+'/metadata/'+str(metaId))
        else:
            self.baseURL('loadbalancers/'+str(lbId)+'/metadata/'+str(metaId))

        #required args
        self.postKeyVar('value',value)

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteMetadata(self, lbId, metaIdList, nodeId=None):
        """
        Remove metadata per a load balancer.
        
        Arguments:
        lbId -- Load Balancer id 
        metaIdList -- Python list containing metadata id's to delete. 
        nodeId -- Node id (Optional)
        
        Returns: status code
        """

        self.postKey = 'metadata'

        if nodeId is not None:
            req = 'loadbalancers/'+str(lbId)+'/nodes/'+str(nodeId)+'/metadata'
        else:
            req = 'loadbalancers/'+str(lbId)+'/metadata'
        if len(metaIdList) == 1:
            self.baseURL(req+'/'+str(metaIdList[0]))
        else:
            self.baseURL(req)
            for metaId in metaIdList:
                self.queryVar('id',str(metaId))

        return apiRequest(self, self.endpoint, "DELETE")

    # }}}

#eof



#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

from Cloud import *

class CloudNetworks(Cloud):
    
    service = "ServersOpenStack"

    def getNetworks(self):
        """
        List the networks configured for a specified tenant ID.
        """
        self.baseURL('os-networksv2')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createNetwork(self, cidr, label):
        """
        Creates a network for the specified tenant ID.
        
        Arguments:
        cidr -- ip block from which to allocate the network
        label -- name of the new network

        Returns:
        JSON results as python dict
        """
        self.baseURL('os-networksv2')
        self.postKey = 'network'
        self.postKeyVar('cidr',cidr)
        self.postKeyVar('label',label)
        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def getNetwork(self, nId):
        """
        Fetches network information with an id input.
        
        Arguments:
        nId -- network id
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('os-networksv2/'+str(nId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getNetworks(self):
        """
        Lists the networks configured for a specified tenant ID.
        
        Arguments:
        nId -- network id
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('os-networksv2')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def deleteNetwork(self, nId):
        """
        Deletes the specified network.
        
        Arguments:
        nId -- network ID
        
        Returns:
        status code
        """
        self.baseURL('os-networksv2/'+str(nId))
        return apiRequest(self, self.endpoint, "DELETE")
        
    def provisionServer(self, server, imageRef, flavorRef, attrDict={}):
        """
        Provision a new server with the specified networks.
        
        Arguments:
        server -- Server name
        imageRef -- image id
        flavorRef -- flavor id(string)
        attrDict -- dictionary the following a combination of the following var/vals:
            networks -- list of network uuid's (inside of uuid key dictionaries) to provision server(s) to (Optional)
            OS-DCF:diskConfig -- AUTO / MANUAL (Optional)
            metadata -- dictionary of key-value meta-data (Optional)
            personality -- list of dictionaries specifiying file path and contents (Optional)
        
        Returns:
        JSON results as python dict
        """

        self.baseURL('servers')
        self.attrDict = attrDict
        self.postKey = 'server'

        #required args
        self.postKeyVar('name',server)
        self.postKeyVar('imageRef',imageRef)
        self.postKeyVar('flavorRef',str(flavorRef))

        #optional args
        self.postKeyDict()

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)

#eof



#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

from Cloud import *

class CloudBS(Cloud):
    
    service = "BlockStorage"

    #Volumes {{{

    def createVolume(self, volume, size, attrDict={}):
        """
        Create a new volume.
        
        Arguments:
        volume -- display_name (ie: vol-001)
        size -- size of volume in GB (ie: 30)

        attrDict -- python list options:
            desc -- display_description
            snapID --  snapshot_id
            volumeType -- volume_type
        
        Returns: status code
        """
        
        self.postKey = 'volume'
        self.attrDict = attrDict
        self.baseURL('volumes')

        #required args
        self.postKeyVar('display_name',volume)
        self.postKeyVar('size',size)

        #optional attrDict args
        self.postKeyDictVar('display_name','name')
        self.postKeyDictVar('display_description','desc')
        self.postKeyDictVar('snapshot_id','snapID')
        self.postKeyDictVar('volume_type','volumeType')

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
         
    def getVolumes(self):
        """
        List all volumes.
        
        Arguments:

        Returns: JSON results as python dict
        """

        self.baseURL('volumes')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
        
    def getVolume(self, id):
        """
        Retrieve volume information.

        Arguments:
        id -- volume id

        Returns: JSON results as python dict
        """

        self.baseURL('volumes/'+str(id))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def deleteVolume(self, id):
        """
        Remove a volume.
        
        Arguments:
        id -- volume id
        
        Returns: status code
        """
        self.baseURL('volumes/'+str(id))
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Volume Types {{{

    def getVolumeTypes(self):
        """
        List all volume types.
        
        Arguments:

        Returns: JSON results as python dict
        """

        self.baseURL('types')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getVolumeType(self, id):
        """
        Retrieve volume type information.

        Arguments:
        id --  volume type id

        Returns: JSON results as python dict
        """

        self.baseURL('types/'+str(id))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Snapshots {{{

    def createSnapshot(self, snapshot, vId, attrDict={}):
        """
        Create a new snapshot.
        
        Arguments:
        snapshot -- snapshot representational name
        vId -- volume id
        attrDict -- python list options:
            force -- (true/false) force a snapshot
            name -- display name
            desc -- display descriptioin
        
        Returns: status code
        """

        self.postKey = 'snapshot'
        self.attrDict = attrDict
        self.baseURL('snapshots')

        #required args
        self.postKeyVar('snapshot',snapshot)
        self.postKeyVar('volume_id',vId)

        #optional attrDict args
        self.postKeyDictVar('force','force')
        self.postKeyDictVar('display_name','name')
        self.postKeyDictVar('display_description','desc')

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def getSnapshots(self):
        """
        List all snapshots.
        
        Arguments:

        Returns: JSON results as python dict
        """
        self.baseURL('snapshots')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getSnapshot(self, id):
        """
        Retrieve snapshot information.

        Arguments:
        id --  snapshot id

        Returns: JSON results as python dict
        """
        self.baseURL('snapshots/'+str(id))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def deleteSnapshot(self, id):
        """
        Remove a snapshot.
        
        Arguments:
        id -- snapshot id
        
        Returns: status code
        """
        self.baseURL('snapshots/'+str(id))
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}

#eof



#!/usr/bin/python

#{{{
# Copyright 2012 Rackspace Hosting
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#   ----
#}}}

import logging
import traceback
import sys

"""  
exception classes
"""

class Error(StandardError):
    """
    Base class for all errors and exceptions
    """
    pass
    
class RSException(Error):
    """
    Raised on a 
    """
    def __init__(self, methodName=None, desc=None):
        if methodName is None: 
            pass
        elif desc is None:
            e = "RSException error from %s" % methodName
            print e
            logging.error(e)
            type_, value_, traceback_ = sys.exc_info()
            tb = traceback.format_exception(type_, value_, traceback_)
            logging.error(tb)
        else:
            e = "RSException error from %s - %s" % (methodName, desc)
            print e
            logging.error(e)
            type_, value_, traceback_ = sys.exc_info()
            tb = traceback.format_exception(type_, value_, traceback_)
            logging.error(tb)

#eof



#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

from Cloud import *

class CloudServersFG(Cloud):
    
    service = "Servers"

    def getLimits(self):
        """
        Gets the current rate and absolute limits for your account.
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('limits') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        

    #Servers{{{

    def getServers(self, details=False, attrDict={}):
        """
        List all servers ([1]Without details = IDs + names only [2]With details = all details)
        
        Arguments:
        details -- true/false  
        """
        if details:
            self.baseURL('servers/detail') 
        else:
            self.baseURL('servers') 
        self.attrDict = attrDict
        self.queryDict()
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getServer(self, sId):
        """
        Fetch a specific server.
        
        Arguments:
        sId -- Server uuid
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('servers/'+str(sId)) 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createServer(self, name, flavorId, imageId, attrDict={}):
        """
        Creates a server.
        
        Arguments:
        name --
        flavorId --
        imageId -- uuid(string)
        attrList -- Python dictionary options:
            metadata --
            personality --

        Returns:
        JSON results as python dict
        """
        self.baseURL('servers') 
        self.postKey = 'server'
        self.attrDict = attrDict

        #required args
        self.postKeyVar('name',name) 
        self.postKeyVar('imageId',imageId) 
        self.postKeyVar('flavorId',flavorId) 

        #optional args
        self.postKeyDict()

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def updateServer(self, sId, attrDict={}):
        """
        Updates the editable attributes for the specified server.
        
        Arguments:
        id -- Server uuid
        attrList -- python dictionary options:
            name --
            adminPass --
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('servers/'+str(sId)) 
        self.postKey = 'server'
        self.attrDict = attrDict
        self.postKeyDict()
        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteServer(self, sId):
        """
        Delete the specified server.
        
        Arguments:
        sId -- Server uuid
        
        Returns:
        status code
        """
        self.baseURL('servers/'+str(sId)) 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Server Addresses {{{

    def getAddresses(self, sId, network=None):
        """
        This operation lists all networks and addresses associated with a specified server.  Specify network for filtering.
        
        Arguments:
        sId -- Server uuid
        network -- Filter by network ie public, private
        
        Returns:
        JSON results as python dict
        """
        if network in ['private','public']:
            self.baseURL('servers/'+str(sId)+'/ips/'+str(network)) 
        else:
            self.baseURL('servers/'+str(sId)+'/ips') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def shareAddress(self, sId, addr, attrDict={}):
        """
        Shares an address to a specifies server.
        
        Arguments:
        sId -- Server uuid
        addr -- 
        attrDict -- python dict options:
            sharedIpGroupId --
            configureServer -- true/false
        
        Returns:
        JSON results as python dict
        """

        self.baseURL('servers/'+str(sId)+'/ips/public/'+str(addr)) 
        self.attrDict = attrDict
        self.postKey = 'shareIp'

        #option args
        self.postKeyDict()

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def unshareAddress(self, sId, addr):
        """
        Shares an address to a specifies server.
        
        Arguments:
        sId -- Server uuid
        addr -- 
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('servers/'+str(sId)+'/ips/public/'+str(addr)) 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Server Actions {{{    

    def _serverAction(self, sId, actionDict={}, returnResults=False):
        """
        Post requested action to api for included uuid.  
        
        Arguments:
        sId -- server uuid
        actionList -- dictionary containing action request info
        """
        self.baseURL('servers/'+str(sId)+'/action') 
        self.postData = actionDict
        if self.postData == {}:
            return apiRequest(self, self.endpoint, "POST", returnResults)
        else:
            return apiRequest(self, self.endpoint, "POST", self.postData, returnResults)
        
    def rebootServer(self, sId, rType="SOFT"):
        """
        Reboots the specified server.
        
        Arguments:
        sId -- server uuid
        rType -- HARD or SOFT(string)
        
        Returns:
        status code
        """
        data = { 'reboot': { 'type': rType.upper() } }
        return self._serverAction(sId, data)
        
    def rebuildServer(self, sId, iId):
        """
        Rebuilds the specified server.
        
        Arguments:
        sId -- server uuid
        iId -- image Id
        
        Returns:
        JSON results as python dict
        """
        data = {'rebuild': {'imageId':iId}}
        #return self._serverAction(sId, data, returnResults=True)
        return self._serverAction(sId, data)
        
    def resizeServer(self, sId, fId, diskConfig='AUTO'):
        """
        Resizes the specified server.
        
        Arguments:
        sId -- server uuid
        fId -- flavor Id
        
        Returns:
        status code
        """
        data = {'resize':{'flavorId':fId}}
        return self._serverAction(sId, data)
        
    def confirmResizeServer(self, sId):
        """
        Confirm a pending resize action.
        
        Arguments:
        sId -- server uuid
        
        Returns:
        status code
        """
        data = {'confirmResize': None}
        return self._serverAction(sId, data)
        
    def revertResizeServer(self, sId):
        """
        Cancels and reverts a pending resize action.
        
        Arguments:
        sId -- server uuid
        
        Returns:
        status code
        """
        data = {'revertResize':None}
        return self._serverAction(sId, data)

    # }}}
    #Flavors{{{

    def getFlavors(self, details=False):
        """
        Fetch info for  all available flavors. Specify details=True to get details.
        
        Arguments:
        details -- Bool, defaults to False.  
        
        Returns:
        JSON results as python dict
        """
        if details:
            self.baseURL('flavors/detail') 
        else:
            self.baseURL('flavors') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getFlavor(self, fId):
        """
        Fetch info for  all available flavors. Specify details=True to get details.
        
        Arguments:
        fId -- flavor Id
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('flavors/'+str(fId)) 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Images{{{

    def getImages(self, details=False):
        """
        Fetch info for  all available images. Specify details=True to get details.
        
        Arguments:
        details -- Bool, defaults to False.  Specifying iId overrides this.
        
        Returns:
        JSON results as python dict
        """
        if details:
            self.baseURL('images/detail') 
        else:
            self.baseURL('images') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getImage(self, iId):
        """
        Fetch an image.
        
        Arguments:
        iId -- image id
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('images/'+str(iId)) 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def deleteImage(self, iId):
        """
        This operation deletes the specified image from the system.
        
        Arguments:
        iId -- image id
        
        Returns:
        status code
        """
        self.baseURL('images/'+str(iId)) 
        return apiRequest(self, self.endpoint, "DELETE")
        
    def createImage(self, sId, name):
        """
        Creates a new image.
        
        Arguments:
        sId -- server id
        name -- image name

        Returns:
        JSON results as python dict
        """

        self.baseURL('images') 
        self.postKey = 'image'

        #required vars
        self.postKeyVar('serverId',sId)
        self.postKeyVar('name',name)

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        

    # }}}
    #Backup Schedules{{{

    def getBSchedule(self, sId):
        """
        Lists the backup schedules for a specified server.
        
        Arguments:
        sId -- server id
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('servers/'+str(sId)+'/backup_schedule') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def setBSchedule(self, sId, attrDict={}):
        """
        Sets (creates/updates) the backup schedule for a specified server.
        
        Arguments:
        sId -- server id
        attrDict -- python dict options:
            enabled -- true/false
            weekly -- ie: SUNDAY
            daily -- ie: H_0400_0600

        Returns:
        JSON results as python dict
        """

        self.baseURL('servers/'+str(sId)+'/backup_schedule') 
        self.postKey = 'backupSchedule'
        self.attrDict = attrDict

        #optional args
        self.postKeyDict()

        return apiRequest(self, self.endpoint, "POST", self.postData)
        
    def disableBSchedule(self, sId):
        """
        Disable the backup schedule a specified server.
        
        Arguments:
        sId -- server id
        
        Returns:
        status code
        """
        self.baseURL('servers/'+str(sId)+'/backup_schedule') 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Shared IP Groups{{{

    def getIPGroups(self, details=False):
        """
        Fetch all current shared IP groups. Toggle details (=True) for more informative results.
        
        Arguments:
        details -- Bool, defaults to False.  Specifying gId overrides this.
        
        Returns:
        JSON results as python dict
        """

        if details:
            self.baseURL('shared_ip_groups/detail') 
        else:
            self.baseURL('shared_ip_groups') 
            #self.baseURL('shared_ip_groups/detail') 

        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getIPGroup(self, gId):
        """
        List IDs and names for shared IP Groups. 
        Specify a gId for details on that IP group or 
        details=True for all details.
        
        Arguments:
        gId -- IP group id (Optional)
        details -- Bool, defaults to False.  Specifying gId overrides this.
        
        Returns:
        JSON results as python dict
        """

        self.baseURL('shared_ip_groups/'+str(gId)) 
        
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def deleteIPGroup(self, gId):
        """
        Deletes a specified IP Group.
        
        Arguments:
        gId -- IP Group id
        
        Returns:
        status code
        """
        self.baseURL('shared_ip_groups/'+str(gId)) 
        return apiRequest(self, self.endpoint, "DELETE")
        
    def createIPGroup(self, gName, sId=None):
        """
        Create a shared IP group.
        
        Arguments:
        gName -- shared ip group name
        sId -- server id

        Returns:
        JSON results as python dict
        """

        self.baseURL('shared_ip_groups')
        self.postKey = 'sharedIpGroup'

        #required args
        self.postKeyVar('name',gName)
        self.postKeyVar('server',sId)

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)

    # }}}

#eof



#!/usr/bin/python

from Auth1 import *
from Auth2 import *
from Cloud import *
from CloudBS import *
from CloudDB import *
from CloudDNS import *
from CloudFiles import *
from CloudLB import *
from CloudMonitor import *
from CloudNetworks import *
from CloudServersFG import *
from CloudServersNG import *


#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

import requests
import yaml
import simplejson as json
import os
import sys
import re
from datetime import datetime
import dateutil.parser
from pytz import utc
import keyring
import keyring.backend
from getpass import getpass
from bin._Globals import *

"""
A module to interface into Rackspace Cloud's Auth 2 api.  

Exported Classes:

Auth2 -- A class that handles the auth and token caching for Rackspace Cloud accounts via Auth 1.1 api.
"""

class Auth2(object):
    """A class that handles the auth and token caching for Rackspace Cloud accounts.
    It only uses api key for auth.  You will be prompted for creds if a .cloudAccount.yml 
    file doesn't exist.  Create the object to initially validate.  Any time an api 
    request is needed, call validate() to verify the token.  
    
    Variables:
    cloudAccount -- List containing the following creds
    username -- Cloud username(string)
    key      -- Cloud api key(string)
    token    -- Cloud cached token(string)
    expires  -- Cloud token expiration time(string)
    tenant   -- Cloud tenant id
    
    cloudAccountUK -- List containing the following UK creds
    username  -- Cloud UK username(string)
    key       -- Cloud UK api key(string)
    token     -- Cloud UK token(string)
    expires   -- Cloud UK token expiration time(string)
    tenant    -- Cloud tenant id
    
    endpoints -- Dictionary containing US and UK Auth endpoints
    
    pd -- Current working directory

    Public Functions:
    __init__ -- Object initialization
    validate -- Validates current token stored in cloudAccount.
    
    Private Functions:
    _auth    -- Authenticate using cloudAccount credentials and endpoints
    _updateResults -- Dumps account and endpoints to file
    _logResults    --
    
    """
    
    cloudAccount = ""
    env = ""
    region = None
    headers = { }
    
    endpoints = dict(
        US="https://identity.api.rackspacecloud.com/v2.0/tokens",
        UK="https://lon.identity.api.rackspacecloud.com/v2.0/tokens"
        )
        
    pd = os.path.dirname(os.path.realpath(__file__))
    
    def __init__(self, env):
        """Initializes Auth1 object and assigns cloudAccount dict based on env and keyring creds.  Calls validate().
        
        Arguments:
        env -- The cloud environment(us,uk)(string).
        """
        
        if re.search("us", env, re.IGNORECASE) or re.search("uk", env, re.IGNORECASE):
            self.env = env.upper()
        self._getKeyring()
        
        self.validate()
        
    def validate(self, force=False):
        """Checks expires time to current time and re-auths if needed.
        
        Arguments:
        force -- Force token re-auth if needed(bool).
        
        Returns:
        True or False indicating an existing valid cached token or re-auth success.
        """ 
        
        if not force:
            if (dateutil.parser.parse(self.cloudAccount['expires']) > datetime.now(utc)):
                if 'X-Auth-Token' not in self.headers:
                    self._setHeaders()
                return True
            else:
                return self._auth()
        else:
            return self._auth()
                    
    def _auth(self):
        """Re-Authenticates to Rackspace Cloud endpoints using data in cloudAccount lists.
        
        Returns:
        True of False indicating success of failure.
        """
        if self.cloudAccount['key'] is None:
            credentials = { "auth":{ "passwordCredentials":{ "username":self.cloudAccount['username'], "password":self.cloudAccount['password'] } } }
        else:
            credentials = { "auth":{ "RAX-KSKEY:apiKeyCredentials":{ "username":self.cloudAccount['username'], "apiKey":self.cloudAccount['key'] } } }
        
        headers = {'content-type': 'application/json'}
        
        try:
            r = requests.post(self.endpoints[self.env], data=json.dumps(credentials), headers=headers)

            if r.status_code == requests.codes.ok:
                self._logResults("Success-", r.text)
                return self._updateResults(r)
            else:
                print r.text
                self._logResults("Error-", r.text)
                return False
        except requests.exceptions.RequestException as error:
            print "Error when processing request"
            #log somewhere
            return False
            
    def _updateResults(self, r):
        """Updates endpoints.yaml file and calls _setKeyring based on successful auth.
        
        Arguments:
        r -- Requests object containing our data
        
        Returns:
        True or False depending on successful yaml updating
        """
        data = json.loads(r.text)
        cloudServices = json.loads(json.dumps(data['access']['serviceCatalog']))
        auth = json.loads(json.dumps(data['access']['token']))
        
        self.cloudAccount['token'] = auth['id']
        self.cloudAccount['expires'] = auth['expires']
        
        self._setKeyring() #store the new token and expiration
        self._setHeaders() #updating the headers  
        
        f = file(os.path.join(self.pd,".cloudServices%s.yml" % self.env), 'w')
        yaml.dump(cloudServices,f, default_flow_style=False)
        f.close()
        
        return True
        
    def _getKeyring(self):
        """
        This method checks for keyring entries for Cloud credentials and prompts for them if not present.
        Should be called with validate() and initialization.
        
        """
        
        username = keyring.get_password('cloud_auth%s' % self.env, 'username')
        key = keyring.get_password('cloud_auth%s' % self.env, 'key')
        password = keyring.get_password('cloud_auth%s' % self.env, 'password')
        token = keyring.get_password('cloud_auth%s' % self.env, 'token')
        expires = keyring.get_password('cloud_auth%s' %self.env, 'expires')
        
        if username is None: #prompt for username and api key
            username = raw_input("%s Username: " % self.env)
            print "Return to enter api key instead"
            password = getpass("%s Password: " % self.env)
            if len(password) < 3:
                key = raw_input("%s Api Key: " % self.env)
            token = "invalid"
            expires = "2012-09-27T12:17:48.000+01:00"
            
        self.cloudAccount = { "username":username,"key":key,"password":password,"token":token,"expires":expires }
        
    def _setKeyring(self):
        """
        This method sets the keyring with the cloudAccount dict.        
        
        """ 
        try:
            keyring.set_password('cloud_auth%s' % self.env, 'username', self.cloudAccount['username'])
            if self.cloudAccount['key'] is None:
                keyring.set_password('cloud_auth%s' % self.env, 'password', self.cloudAccount['password'])
            else:
                keyring.set_password('cloud_auth%s' % self.env, 'key', self.cloudAccount['key'])
            keyring.set_password('cloud_auth%s' % self.env, 'token', self.cloudAccount['token'])
            keyring.set_password('cloud_auth%s' % self.env, 'expires', self.cloudAccount['expires'])
        except keyring.backend.PasswordError:
            print "Error storing credentials"
            #log somewhere
    
    def _setHeaders(self, noType=False):
        """
        Updates json headers with latest token.
        """
        
        if noType:
            self.headers = { 'X-Auth-Token': '%s' % self.cloudAccount['token']}
        else:
            self.headers = { 'X-Auth-Token': '%s' % self.cloudAccount['token'],'content-type': 'application/json'}
        
    def _logResults(self, status, results):
        """
        Uses logger to log to file
        """
        pass



#!/usr/bin/python

#{{{
# Copyright 2012 Rackspace Hosting
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#   ----
#}}}

import requests
import re
import os
import simplejson as json
import traceback
import sys
import logging

#make log dir if doesn't exist
if not os.path.exists("log"):
    os.makedirs("log")
    
#setup root logger. Valid levels (DEBUG,INFO,WARNING,ERROR,CRITICAL)
logging.basicConfig(
    filename='log/RSCloud.log',
    level=logging.DEBUG,
    format='%(levelname)s %(name)s %(asctime)s %(message)s'
    )

#setup scheduler logger.  Logging levels (DEBUG,INFO,WARNING,ERROR).
log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)

def apiRequest(obj, endpoint, apiType, data=None, returnResults=False, dataType='json'):
    """
    Makes api request to the specified endpoint with the provided details
    
    Arguments:
    obj -- Calling object
    endpoint -- Api endpoint(string)
    apiType -- GET, POST, PUT, DELETE etc.(string)
    data -- Python dict data for endpoint
    returnResults -- True: returns JSON results as Python Dict. False returns status code
    
    Returns:
    Results or status code
    """
    try:
        obj.auth.validate()
        
        if re.search("GET", apiType.upper()):
            r = requests.get(endpoint, headers=obj.auth.headers)
        elif re.search("DELETE", apiType.upper()):
            r = requests.delete(endpoint, headers=obj.auth.headers)
        elif re.search("HEAD", apiType.upper()):
            r = requests.head(endpoint, headers=obj.auth.headers)
        elif re.search("POST", apiType.upper()) and data is not None:
            r = requests.post(endpoint, data=json.dumps(data), headers=obj.auth.headers)
        elif re.search("POST", apiType.upper()) and data is None:
            r = requests.post(endpoint, headers=obj.auth.headers)
        elif re.search("PUT", apiType.upper()) and data is None:
            r = requests.put(endpoint, headers=obj.auth.headers)
        elif re.search("PUT", apiType.upper()) and data is not None and dataType == "json":
            r = requests.put(endpoint, data=json.dumps(data), headers=obj.auth.headers)
        elif re.search("PUT", apiType.upper()) and data is not None and dataType == "file":
            r = requests.put(endpoint, data=data, headers = obj.auth.headers)
        else:
            raise RSException('apiRequest', 'Invalid api type or data')
        
        if re.search("HEAD", apiType.upper()):
            resultType='headers'
            results = r.headers
        elif returnResults:
            if returnResults == "raw":
                resultType='status_code'
                results = r.status_code
            elif returnResults == "headers":
                resultType='headers'
                results = r.headers
            else:
                resultType='json'
                results = json.loads(json.dumps(json.loads(r.text)))
        else:
            resultType = 'status_code'
            results = r.status_code
        
        if int(r.status_code) >= 200 and int(r.status_code) <= 299:
            logging.debug("Success: %s" % results)    
            if returnResults == "raw":
                results = r.content
            elif returnResults == "headers":
                results = r.headers
            return results
        else:
            log.debug("error: %s" % resultType)
            log.debug("postData:")
            log.debug(obj.postData)
            log.debug("endpoint:")
            log.debug(obj.endpoint)
            log.debug(results)
            logging.error("Error: %s" % results)
            return results
    except requests.exceptions.RequestException as error:
        #log the error
        e = "Error when processing request %s to %s - check logs" % (apiType, endpoint)
        print "---"
        print e
        print "postData:"
        print obj.postData
        print "endpoint:"
        print obj.endpoint
        print "---"
        logging.error(e)
        #format the trace and log
        type_, value_, traceback_ = sys.exc_info()
        tb = traceback.format_exception(type_, value_, traceback_)
        logging.error(tb)
        #re-raise
        raise Exception(e)
    except:
        e = "Unexpected error from request %s to %s - check logs" % (apiType, endpoint)
        print e
        logging.error(e)
        type_, value_, traceback_ = sys.exc_info()
        tb = traceback.format_exception(type_, value_, traceback_)
        logging.error(tb)
        raise Exception(e)

#eof




#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

import requests
import yaml
import simplejson as json
import os
import sys
import re
from datetime import datetime
import dateutil.parser
from pytz import utc
import keyring
import keyring.backend
from bin._Globals import *

"""
A module to interface into Rackspace Cloud's Auth 1.1 api.  

Exported Classes:

Auth1 -- A class that handles the auth and token caching for Rackspace Cloud accounts via Auth 1.1 api.
"""

class Auth1(object):
    """A class that handles the auth and token caching for Rackspace Cloud accounts.
    It only uses api key for auth.  You will be prompted for creds if a .cloudAccount.yml 
    file doesn't exist.  Create the object to initially validate.  Any time an api 
    request is needed, call validate() to verify the token.  
    
    Variables:
    cloudAccount -- List containing the following creds
    username -- Cloud username(string)
    key      -- Cloud api key(string)
    token    -- Cloud cached token(string)
    expires  -- Cloud token expiration time(string)
    tenant   -- Cloud tenant id
    
    cloudAccountUK -- List containing the following UK creds
    username  -- Cloud UK username(string)
    key       -- Cloud UK api key(string)
    token     -- Cloud UK token(string)
    expires   -- Cloud UK token expiration time(string)
    tenant    -- Cloud tenant id
    
    endpoints -- Dictionary containing US and UK Auth endpoints
    
    pd -- Current working directory

    Public Functions:
    __init__ -- Object initialization
    validate -- Validates current token stored in cloudAccount.
    
    Private Functions:
    _auth    -- Authenticate using cloudAccount credentials and endpoints
    _updateResults -- Dumps account and endpoints to file
    _logResults    --
    _logError      --
    
    """
    
    cloudAccount = ""
    env = ""
    region = None
    headers = { }
    
    endpoints = dict(
        US="https://identity.api.rackspacecloud.com/v1.1/auth",
        UK="https://lon.identity.api.rackspacecloud.com/v1.1/auth"
        )
        
    pd = os.path.dirname(os.path.realpath(__file__))
    
    def __init__(self, env):
        """Initializes Auth1 object and assigns cloudAccount dict based on env and keyring creds.  Calls validate().
        
        Arguments:
        env -- The cloud environment(us,uk)(string).
        """
        
        if re.search("us", env, re.IGNORECASE) or re.search("uk", env, re.IGNORECASE):
            self.env = env.upper()
        self._getKeyring()
        
        self.validate()
        
    def validate(self, force=False):
        """Checks expires time to current time and re-auths if needed.
        
        Arguments:
        force -- Force token re-auth if needed(bool).
        
        Returns:
        True or False indicating an existing valid cached token or re-auth success.
        """ 
        
        if not force:
            if dateutil.parser.parse(self.cloudAccount['expires']) > datetime.now(utc):
                self._setHeaders()
                return True
            else:
                return self._auth()
        else:
            return self._auth()
                    
    def _auth(self):
        """Re-Authenticates to Rackspace Cloud endpoints using data in cloudAccount lists.
        
        Returns:
        True of False indicating success of failure.
        """
        
        credentials = { "credentials":{ "username":self.cloudAccount['username'], "key":self.cloudAccount['key'] } }
        headers = {'content-type': 'application/json'}
        
        try:
            r = requests.post(self.endpoints[self.env], data=json.dumps(credentials), headers=headers)

            if r.status_code == requests.codes.ok:
                self._logResults("Success-", r.text)
                return self._updateResults(r)
            else:
                print r.text
                self._logResults("Error-", r.text)
                return False
        except requests.exceptions.RequestException as error:
            print "Error when processing request"
            #log somewhere
            return False
            
    def _updateResults(self, r):
        """Updates cloudAccount.yaml and endpoints.yaml files based on successful auth.
        
        Arguments:
        r -- Requests object containing our data
        
        Returns:
        True or False depending on successful yaml updating
        """
        #should be in a try block
        data = json.loads(r.text)
        cloudServices = json.loads(json.dumps(data['auth']['serviceCatalog']))
        auth = json.loads(json.dumps(data['auth']['token']))
        
        self.cloudAccount['token'] = auth['id']
        self.cloudAccount['expires'] = auth['expires']
        
        self._setKeyring() #store the new token and expiration
        self._setHeaders() #stores the new token headers
        
        f = file(os.path.join(self.pd,".cloudServices%s.yml" % self.env), 'w')
        yaml.dump(cloudServices,f, default_flow_style=False)
        f.close()
        
        return True
        
    def _getKeyring(self):
        """
        This method checks for keyring entries for Cloud credentials and prompts for them if not present.
        Should be called with validate() and initialization.
        
        """
        
        username = keyring.get_password('cloud_auth%s' % self.env, 'username')
        key = keyring.get_password('cloud_auth%s' % self.env, 'key')
        #password = keyring.get_password('cloud_auth%s' % self.env, 'password')
        token = keyring.get_password('cloud_auth%s' % self.env, 'token')
        expires = keyring.get_password('cloud_auth%s' % self.env, 'expires')
        
        if username is None: #prompt for username and api key
            username = raw_input("%s Username: " % self.env)
            key = raw_input("%s Api key: " % self.env)
            token = "invalid"
            expires = "2012-09-27T12:17:48.000+01:00"
            
        if key is None and isinstance(self.cloudAccount, str):
            key = raw_input("%s Api key: " % self.env)
            
        self.cloudAccount = { "username":username,"key":key,"token":token,"expires":expires }
        
    def _setKeyring(self):
        """
        This method sets the keyring with the cloudAccount dict.        
        
        """ 
        try:
            keyring.set_password('cloud_auth%s' % self.env, 'username', self.cloudAccount['username'])
            keyring.set_password('cloud_auth%s' % self.env, 'key', self.cloudAccount['key'])
            keyring.set_password('cloud_auth%s' % self.env, 'token', self.cloudAccount['token'])
            keyring.set_password('cloud_auth%s' % self.env, 'expires', self.cloudAccount['expires'])
        except keyring.backend.PasswordError:
            print "Error storing credentials"
            #log somewhere

    def _setHeaders(self, noType=False):
        """
        Updates json headers with latest token.
        """

        if noType:
            self.headers = { 'X-Auth-Token': '%s' % self.cloudAccount['token']}
        else:
            self.headers = { 'X-Auth-Token': '%s' % self.cloudAccount['token'],'content-type': 'application/json'}            
    
    def _logResults(self, status, results):
        """
        Uses logger to log to file
        """
        pass



#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

from Cloud import *

class CloudFiles(Cloud):
    """
    A class to interface into Rackspace Cloud Files.
    """

    service = "Files"

    #Storage Account Services{{{

    def getAccountMeta(self):
        """
        Retrieves the number of containers and the total bytes stored in Cloud Files for the account. 
        
        Returns:
        Results header as string
        """
        return apiRequest(self, self.publicURL, "HEAD")
        
    def setAccountMeta(self, metaDict):
        """
        Allows for metadata entries to an account(TempURL or form posts etc)
        
        Arguments:
        metaDict -- Python dictionary of key/value pairs to be added.
        
        Returns:
        status code
        """
        
        try:
            self.auth.headers = dict(self.auth.headers.items() + metaDict.items())
            return apiRequest(self, self.publicURL, "POST")
        except:
            raise RSException('setAccountMeta')
        
    def getContainers(self, attrDict={}): #limit=None, marker=None, format='json'):
        """
        Lists the available containers in the Cloud Files account.
        
        Arguments:
        attrDict --
            limit -- For an integer value n, limits the number of results to n values. (Optional)
            marker -- Given a string value x, return object names greater in value than the marker. (Optional)
            format -- Specify either json or xml to return the respective serialized response. (Optional)
        """
        
        self.baseURL('')
        if format not in attrDict:
            attrDict['format'] = 'json'
        self.attrDict = attrDict
        self._queryDict()
        #self.queryVar('format',format)
        #if limit is not None:
        #    self.queryVar('limit',limit)
        #if marker is not None:
        #    self._queryVar('marker',marker)
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    #}}}
    #Storage Container Services{{{

    def createContainer(self, container, metaDict=None):
        """
        Create a container at the root of the CF account.
        
        Arguments:
        container -- container name (string)
        metaDict -- A python dictionary with key/value pairs for metadata tags and desc etc.  
            Must be valid UTF-8 http header and take the format X-Container-Meta- etc.
        
        Returns:
        status code
        """
        self._baseURL(container)
        try:
            if metaDict is not None:
                self.auth.validate() #make sure new headers don't get overwritten
                self.auth.headers = dict( self.auth.headers.items() + metaDict.items() )
            return apiRequest(self, self.endpoint, "PUT")

        except:
            raise RSException('setContainer')
        
    def deleteContainer(self, container):
        """
        Permanently remove a container.  Must be empty
        
        Arguments:
        container --
        
        Returns:
        status code
        """
        self._baseURL(container)
        return apiRequest(self, self.endpoint, "DELETE")
        
    def getContainerMeta(self, container):
        """
        Retrieves metadata for a specified container.
        
        Arguments:
        container --
        
        Returns:
        Header info as string
        """
        self._baseURL(container)
        return apiRequest(self, self.endpoint, "HEAD")
        
    def setContainerMeta(self, container, metaDict):
        """
        Create or update metadata on a container.
        
        Arguments:
        container --
        metaDict -- A python dictionary with key/value pairs for metadata tags and desc etc.  
            Must be valid UTF-8 http header and take the format X-Container-Meta- etc.
        
        Returns:
        status code            
        """
        self._baseURL(container)
        try:
            if metaDict is not None:
                self.auth.validate() #make sure new headers don't get overwritten
                self.auth.headers = dict( self.auth.headers.items() + metaDict.items() )
            return apiRequest(self, self.endpoint, "POST")
        except:
            raise RSException('setContainerMeta')
        
    def getObjects(self, container, attrDict={}): 
        """
        Retrieves a list of objects stored in the container. Additionally, there are a number of optional 
        query parameters that can be used to refine the list results.
        
        Arguments:
        container --
        attrDict --
            limit -- For an integer value n, limits the number of results to n values. (Optional)
            marker -- Return object names greater in value than the specified marker. (Optional)
            prefix -- Causes the results to be limited to object names beginning with the substring (Optional)
            format -- Specify either json or xml to return the respective serialized response. Default json (Optional)
            path -- Return the object names nested in the pseudo path.  (Optional)
            delimiter -- Return all the object names nested in the container with given delimiter (Optional)
        
        Returns:
        JSON results as python dict
        """
        self._baseURL(container)
        if format not in attrDict:
            attrDict['format'] = 'json'
        self.attrDict = attrDict
        self._queryDict()
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    #}}}
    #Storage Object Services{{{

    def getObject(self, container, myObject, headersList=None):
        """
        Retrieves CF objects data.
        
        Arguments:
        container --
        myObject --
        headersList -- HTTP headers as documented in RFC 2616 as key/value in python dictionary. See api docs.
        
        Returns:
        Binary content on success or status code on failure.
        """
        self._baseURL(self.uri(container)+"/"+self.uri(myObject))
        try:
            if headersList is not None:
                self.auth.validate()
                self.auth._setHeaders(noType=True)
                self.auth.headers = dict( self.auth.headers.items() + headersList.items() )
            return apiRequest(self, self.endpoint, "GET", returnResults='raw')
        except:
            raise RSException('getObject')
        
    def createObject(self, container, myObject, data, headersList=None):
        """
        PUT object into CF container.
        
        Arguments:
        container --
        myObject -- Object name(string)
        data -- File object
        headersList -- HTTP headers as documented in RFC 2616 as key/value in python dictionary. See api docs. (Optional)
        
        Returns:
        Header results
        """
        self.baseURL(self.uri(container)+"/"+self.uri(myObject))
        self.postData = data
        try:
            if headersList is not None:
                self.auth.validate()
                self.auth._setHeaders(noType=True)
                self.auth.headers = dict( self.auth.headers.items() + headersList.items() )
            return apiRequest(self, self.endpoint, "PUT", self.postData, returnResults='headers', dataType='file')
        except:
            raise RSException('setObject')
        
    def copyObject(self, sourceContainer, sourceObject, destContainer, destObject):
        """
        Copy object into CF container.
        
        Arguments:
        sourceContainer --
        sourceObject -- Name(string)
        destContainer -- 
        destinationObject -- Name(string)
        
        Returns:
        status code
        """
        self._baseURL(self.uri(destContainer)+"/"+self.uri(destObject))
        self.auth.validate()
        self.auth._setHeaders(noType=True)
        headers = { 'X-Copy-From': '/%s/%s' % (self.uri(sourceContainer), self.uri(sourceObject)), 'Content-Length': '0'}
        self.auth.headers = dict( self.auth.headers.items() + headers.items() )
        return apiRequest(self, self.endpoint, "PUT")
        
    def deleteObject(self, container, myObject):
        """
        Delete CF object.
        
        Arguments:
        container --
        myObject --
        
        Returns:
        status code
        """
        self.baseURL(self.uri(container)+"/"+self.uri(myObject))
        return apiRequest(self, self.endpoint, "DELETE")
        
    def getObjectMeta(self, container, myObject):
        """
        Retrieves object metadata.
        
        Arguments:
        container --
        myObject --
        
        Returns:
        Header info
        """
        self.baseURL(self.uri(container)+"/"+self.uri(myObject))
        return apiRequest(self, self.endpoint, "HEAD")
        
    def setObjectMeta(self, container, myObject, metaDict):
        """
        Replaces user defined metadata with supplied.
        
        Arguments:
        container --
        myObject --
        metaDict -- Python dictionary with key/values to be added.   Keys must be prefixed with X-Object-Meta
        
        Returns:
        status code
        """
        self.baseURL(self.uri(container)+"/"+self.uri(myObject))
        try:
            self.auth.validate()
            self.auth.headers = dict(self.auth.headers.items() + metaDict.items() )
            return apiRequest(self, self.endpoint, "POST")
        except:
            raise RSException("setObjectMeta")

    #}}}
    #CDN Services{{{

    def setCDNContainer(self, container, enabled=True, ttl=None):
        """
        Enable or disable CDN services on a container.
        
        Arguments:
        container --
        enabled -- bool (Optional)
        ttl -- How long till CDN cache expires.   The default TTL value is 259200 seconds, or 72 hours (Optional)
        
        Returns:
        headers as string
        """
        headers = { 'X-CDN-Enabled': str(enabled), 'X-Log-Retention': str(enabled) }
        if ttl is not None:
            headers['ttl'] = ttl
        self._basecdnURL(container)
        return apiRequest(self, self.endpoint, "PUT", returnResults='headers')
        
    def getCDNContainers(self, attrDict={}): 
        """
        Container list request from the CDN service.  It will default to showing public and private containers.
        
        Arguments:
        attrDict -- 
            limit -- Limits the number of results to n values. (Optional)
            marker -- Return object names greater in value than the specified marker. (Optional)
            format -- either json or xml for serialized output.  Default json (Optional)
            enabledOnly -- If True, only shows CDN enabled containers (Optional)
        
        Returns:
        JSON results as python dict
        """
        self.basecdnURL('')
        if format not in attrDict:
            attrDict['format'] = 'json'
        self.attrDict = attrDict
        self.queryDict()
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def setCDNContainerMeta(self, container, metaDict):
        """
        Adjust CDN containers attributes.
        
        Arguments:
        container --
        metaDict -- Python dictionary with key/value pairs to append to container metadata
        
        Returns:
        Headers as string
        """
        self._basecdnURL(container)
        try:
            self.auth.headers = dict( self.auth.headers.items() + metaDict.items() )
            return apiRequest(self, self.endpoint, "POST", returnResults="headers")
        except:
            raise RSException('setCDNContainerMeta')
            
    def getCDNContainerMeta(self, container):
        """
        Returns metadata for specified CDN container.
        
        Arguments:
        container --
        
        Returns:
        Headers as string
        """
        self._basecdnURL(container)
        return apiRequest(self, self.endpoint, "HEAD")
        
    def deleteCDNObject(self, container, myObject):
        """
        Purge CDN cache for a published object. Limit 25/day
        
        Arguments:
        container --
        myObject --
        
        Returns:
        status code
        """
        self._basecdnURL(self.uri(container)+"/"+self.uri(myObject))
        return apiRequest(self, self.endpoint, "DELETE")

    #}}}

#eof            



#!/usr/bin/python
        
# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

from yaml import load
from simplejson import loads,dumps
from Auth2 import *
from bin._Globals import *
from bin._Exceptions import *
from urllib import quote

#setup stdout debug logging (True/False) 
#NOTE: This is just for showing in stdout clearly where the test outputs start and stop.
try:
    with open('soLog') as f:
        soLog = True
except IOError as e:
    soLog = False

class Cloud(object):
  
    #attributes{{{

    publicURL = None
    internalURL = None
    cdnURL = None
    auth = None
    env = None
    region = None
    pd = os.path.dirname(os.path.realpath(__file__))
    endpoint = None
    postData = {}
    soLogFlg = None
    
    #}}}
    def __init__(self, env, region=None):
        """
        Initialization of Cloud object.  Defaults to LON or ORD based on env.
        
        Arguments:
        env -- us or uk etc
        region -- LON, DFW or ORD
        """

        if not re.search("us", env, re.IGNORECASE) and not re.search("uk", env, re.IGNORECASE):
            raise RSException("__init__", "Valid env needs to be us/uk")
        
        self.auth = Auth2(env) #re-validate
        self.env = env.upper()
        
        #soLog (temporary stdout handling)
        self.soLogFlg = soLog

        try:
            f = file(os.path.join(self.pd,".cloudServices%s.yml" % self.env), 'r')
            services = load(f)
            f.close
        
            for service in services:
                if service['name'] == "cloud"+self.service:
                    if region is None:
                        self.publicURL = service['endpoints'][0]['publicURL']
                        if 'internalURL' in service['endpoints'][0]:
                            self.internalURL = service['endpoints'][0]['internalURL']
                    else:
                        self.region = region
                        for endpoint in service['endpoints']:
                            if 'region' not in endpoint or re.search(region, endpoint['region'], re.IGNORECASE):
                                self.publicURL = endpoint['publicURL']
                                if 'internalURL' in endpoint:
                                    self.internalURL = endpoint['internalURL']
                #if cloudFiles service, grab cdnURL
                if self.service == "Files" and service['name'] == 'cloudFilesCDN':
                    if region is None:
                        self.cdnURL = service['endpoints'][0]['publicURL']
                    else:
                        for endpoint in service['endpoints']:
                            if re.search(region, endpoint['region'], re.IGNORECASE):
                                self.cdnURL = endpoint['publicURL']
        except:
            raise RSException("__init__")
            
        if self.publicURL is None:
            raise RSException("__init__", "publicURL value cannot be None") 
        
    def baseURL(self, base):
        """
        Updates endpoint with publicURL and base.
        """
        self.postData={}
        #check to see if the full url is there (ie: Async cmd), otherwise use publicURL
        if re.search('https://',str(base).lower()) or re.search('http://',str(base).lower()):
            self.endpoint = "%s?" % (str(base))
        else:
            self.endpoint = "%s/%s?" % (self.publicURL,str(base))
        
    def _baseURL(self, base):
        """
        Updates endpoint with publicURL and base. (uri)
        """
        base = self.uri(base)
        self.baseURL(base)
        
    def basecdnURL(self, base):
        """
        Updates endpoint with cdnURL and base.
        """
        self.postData={}
        #check to see if the full url is there (ie: Async cmd), otherwise use publicURL
        if re.search('https://',str(base).lower()) or re.search('http://',str(base).lower()):
            self.endpoint = "%s?" % (str(base))
        else:
            self.endpoint = "%s/%s?" % (self.cdnURL,str(base))
        
    def _basecdnURL(self, base):
        """
        Updates endpoint with publicURL and base. (uri)
        """
        base = self.uri(base)
        self.basecdnURL(base)
        
    def queryVar(self, key, val):
        """
        Updates endpoint with key and val.
        """
        if val is not None:
            self.endpoint = self.endpoint + "%s=%s&" % (str(key),str(val))
        
    def _queryVar(self, key, val):
        """
        Updates endpoint with key and val. (uri)
        """
        val = self.uri(val)
        self.queryVar(var,val)
        
    def queryDictVar(self,key,attrKey=None):
        """
        Updates endpoint with a dictionary key and the key's val.
        """
        if attrKey is None:
            attrKey=key
        if attrKey in self.attrDict and self.attrDict[attrKey] is not None:
            self.endpoint = "%s=%s&" % (str(key),str(self.attrDict[attrKey]))
        
    def postVar(self,key,val):
        """
        Updates post data with key and val.
        """
        if val is not None:
            self.postData[str(key)] = val
        
    def postDictVar(self,key,attrKey=None):
        """
        Updates post data with a dictionary key and the key's val.
        """
        if attrKey is None:
            attrKey = key
        if attrKey in self.attrDict and self.attrDict[attrKey] is not None:
            self.postData[str(key)] = str(self.attrDict[attrKey])
    def postDict(self):
        """
        Updates post data with all discovered dictionary keys + vals.
        """
        if self.attrDict is not None:
            for key,value in self.attrDict.items():
                if value is not None:
                    self.postData[str(key)] = value
        
    def postKeyVar(self,key,val):
        """
        Updates post data with key and val.
        """
        if str(self.postKey) not in self.postData:
            self.postData[str(self.postKey)] = {}
        self.postData[str(self.postKey)][str(key)] = val
        
    def postKeyDictVar(self,key,attrKey=None):
        """
        Updates post data with a dictionary key and the key's val.
        """
        if attrKey is None:
            attrKey = key
        if str(self.postKey) not in self.postData:
            self.postData[str(self.postKey)] = {}
        if self.attrDict is not None and attrKey in self.attrDict and self.attrDict[attrKey] is not None:
            self.postData[str(self.postKey)][str(key)] = self.attrDict[attrKey]
    def postKeyDict(self):
        """
        Updates post data with all discovered dictionary keys + vals.
        """
        #print "fljljl"
        #print "---"
        #print self.attrDict
        #print "---"
        #print "ljsdljfsljsdfl"
        if self.attrDict is not None and self.attrDict != {}:
            if str(self.postKey) not in self.postData:
                self.postData[str(self.postKey)] = {}
            if self.attrDict is not None:
                for key,value in self.attrDict.items():
                    if value is not None:
                        self.postData[str(self.postKey)][str(key)] = value
        
    def postKeyListDict(self):
        """
        Updates post data with a list of all discovered dictionaries and their keys + vals.
        """
        if str(self.postKey) not in self.postData:
            self.postData[str(self.postKey)] = []
        if self.attrList is not None:
            for d in self.attrList:
                tmpDict = {} 
                for key,value in d.items():
                    if value is not None:
                        tmpDict[str(key)] = value
                self.postData[str(self.postKey)].append(tmpDict)
        
    def queryDict(self):
        """
        Updates endpoint with all discovered dictionary keys + vals.
        """
        if self.attrDict is not None:
            for key,value in self.attrDict.items():
                self.endpoint = self.endpoint + "%s=%s&" % (str(key),str(value))
        
    def _queryDict(self):
        """
        Updates endpoint with all discovered dictionary keys + vals. (uri)
        """
        if self.attrDict is not None:
            for key,value in self.attrDict.items():
                value = self.uri(value)
                self.endpoint = self.endpoint + "%s=%s&" % (str(key),str(value))
        
    def uri(self,val):
        """
        Convert string to uri compliance.
        """
        return unicode(quote(str(val)), 'utf-8')
        
    def so_chk1(self):
        """
        For testing.
        """
        if self.soLogFlg:
            print "endpoint: "+self.endpoint 
            print "postData: "+str(self.postData)

#eof



#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

from Cloud import *

class CloudMonitor(Cloud):
    
    service = "Monitoring"
            
    #Account {{{

    def getAccount(self):
        """
        Returns account information.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('account') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def updateAccount(self, metaDict={}, token=None):
        """
        Update properties on an account.
        
        Arguments:
        metaDict - metadata key-value pairs in form of python dictionary (Optional)
        token -- webhook token (Optional)
        
        Returns: status code
        """

        self.baseURL('account') 
        self.attrDict = metaDict
        self.postKey = 'metadata'

        #optional args
        self.postVar('webhook_token',token)
        self.postKeyDict()

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def getLimits(self):
        """
        Returns account resource limits.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('limits') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getAudits(self):
        """
        Lists audits for this account.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('audits') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Entities {{{

    def getEntities(self):
        """
        Lists the entities for this particular account.
        
        Arguments:
        
        Returns: JSON results as python dict
        
        """
        endpoint = self.publicURL + "/entities"
        return apiRequest(self, endpoint, "GET", returnResults=True)
        
    def getEntity(self, eId):
        """
        Retrieves the current state of an entity.
        
        Arguments:
        eId -- entity id

        Returns: JSON results as python dict
        """
        self.baseURL('entities/'+str(eId)) 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createEntity(self, label, attrDict={}):
        """
        Creates a new entity.
        
        Arguments:
        label -- 
        attrDict -- python dictionary of arg options:
            agent_id --  (Optional)
            ip_addresses - ip's in form of python dictionary (Optional)
            metadata - metadata key-value pairs in form of python dictionary (Optional)
        
        Returns: status code
        """
        
        self.baseURL('entities') 
        self.attrDict = attrDict

        #required args
        self.postVar('label',label)

        #optional args
        self.postDict()

        return apiRequest(self, self.endpoint, "POST", self.postData)
        
    def updateEntity(self, eId, label, attrDict={}):
        """
        Updates an entity specified by the entityId (id).
        
        Arguments:
        eId -- entity id
        label -- (Optional)
        attrDict -- python dictionary of arg options:
            agent_id --  (Optional)
            ip_addresses - ip's in form of python dictionary (Optional)
            metadata - metadata key-value pairs in form of python dictionary (Optional)
        
        Returns: status code
        """

        self.baseURL('entities/'+str(eId)) 
        self.attrDict = attrDict

        #required args
        self.postVar('label',label)

        #optional args
        self.postDict()

        #return apiRequest(self, self.endpoint, "PUT", self.postData, returnResults=True)
        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteEntity(self, eId):
        """
        Delete an entity.
        
        Arguments:
        eId -- entity id
        
        Returns: status code
        """
        self.baseURL('entities/'+str(eId)) 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Checks {{{

    def createCheck(self, entityId, attrDict={}, testMode=False):
        """
        Creates (or test creates) a new check associated with an entity.
        
        Arguments:
        entityId -- 
        attrDict -- python dictionary of options:
            type -- remote.dns / remote.ssh / remote.smtp / remote.http /
                remote.tcp / remote.ping / remote.ftp-banner / remote.imap-banner /
                remote.pop3-banner / remote.smtp-banner / remote.postgresql-banner /
                remote.telnet-banner / remote.mysql-banner /
                remote.mssql-banner  (Required)
            details -- (Optional)
            disabled -- (Optional)
            label -- (Optional)
            metadataDict -- metadata python dict of key/value pairs (Optional)
            period -- (Optional)
            timeout -- (Optional)
            monitoringZonesPoll -- (for remote checks) (Optional)
            targetAlias -- (for remote checks) (Optional)
            targetHost -- (for remote checks) (Optional)
            targetResolver -- (for remote checks) (Optional)
        testMode -- true/false (default=True)
        
        Returns: status code
        """

        if testMode:
            self.baseURL('entities/'+str(entityId)+'/test-check') 
        else:
            self.baseURL('entities/'+str(entityId)+'/checks') 
        self.attrDict = attrDict

        #required + optional args
        self.postDict()

        if testMode:
            return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        else:
            return apiRequest(self, self.endpoint, "POST", self.postData)
        
    def testCheck(self, entityId, attrDict={}):
        """
        Tests the creation of a new check associated with an entity.
        
        Arguments:
        entityId -- entity id
        attrDict -- python dictionary:
            type -- remote.dns / remote.ssh / remote.smtp / remote.http /
                remote.tcp / remote.ping / remote.ftp-banner / remote.imap-banner /
                remote.pop3-banner / remote.smtp-banner / remote.postgresql-banner /
                remote.telnet-banner / remote.mysql-banner / remote.mssql-banner  
            details -- (Optional)
            disabled -- (Optional)
            label -- (Optional)
            metadataDict -- metadata python dict of key/value pairs (Optional)
            period -- (Optional)
            timeout -- (Optional)
            monitoringZonesPoll -- (for remote checks) (Optional)
            targetAlias -- (for remote checks) (Optional)
            targetHost -- (for remote checks) (Optional)
            targetResolver -- (for remote checks) (Optional)
        
        Returns: status code
        """
        return self.createCheck(entityId, attrDict, True)
        
    def getCheck(self, entityId, checkId):
        """
        Returns the specified check.
        
        Arguments:
        entityId -- 
        checkId -- 
        
        Returns: status code.
        """
        self.baseURL('entities/'+str(entityId)+'/checks/'+str(checkId)) 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getChecks(self, entityId):
        """
        Lists checks associated with an entityId.
        
        Arguments:
        entityId --
        
        Returns: JSON results as python dict
        """
        self.postData={}
        self.baseURL('entities/'+str(entityId)+'/checks') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def testRunCheck(self, entityId, checkId):
        """
        Test a check inline.
        
        Arguments:
        entityId -- 
        checkId -- 
        
        Returns: status code.
        """
        self.baseURL('entities/'+str(entityId)+'/checks/'+str(checkId)+'/test') 
        return apiRequest(self, self.endpoint, "POST", returnResults=True)
        
    def updateCheck(self, entityId, checkId, attrDict={}):
        """
        Updates a check associated with an entity.
        
        Arguments:
        entityId -- 
        attrDict -- python dictionary or optional args:
            type -- remote.dns / remote.ssh / remote.smtp / remote.http /
                remote.tcp / remote.ping / remote.ftp-banner / remote.imap-banner /
                remote.pop3-banner / remote.smtp-banner / remote.postgresql-banner /
                remote.telnet-banner / remote.mysql-banner / remote.mssql-banner   (Required)
            details -- (Optional)
            disabled -- (Optional)
            label -- (Optional)
            metadata -- metadata python dict of key/value pairs (Optional)
            period -- (Optional)
            timeout -- (Optional)
            monitoringZonePoll -- (for remote checks) (Optional)
            targetAlias -- (for remote checks) (Optional)
            targetHost -- (for remote checks) (Optional)
            targetResolver -- (for remote checks) (Optional)
        
        Returns: status code
        """
        self.baseURL('entities/'+str(entityId)+'/checks/'+str(checkId)) 
        self.attrDict = attrDict
        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteCheck(self, entityId, checkId):
        """
        Deletes a check.
        
        Arguments:
        entityId -- 
        checkId -- 
        
        Returns: status code
        """
        self.baseURL('entities/'+str(entityId)+'/checks/'+str(checkId)) 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Check Types {{{

    def getCheckType(self, id):
        """
        Returns the specified check type.
        
        Arguments:
        id -- check type id
        
        Returns: status code.
        """
        self.baseURL('check_types/'+str(id)) 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getCheckTypes(self):
        """
        List available check types.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('check_types') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Alarms {{{

    def createAlarm(self, entityId, checkId, planId, attrDict={}, testMode=False):
        """
        Create (or test run the creation of) a(n) new alarm.
        
        Arguments:
        entityId --
        checkId --
        planId -- notification plan id
        attrDict -- python dict options:
            criteria -- (Optional)
            disabled -- (Optional) (true/false)
            label -- (Optional)
            metadata -- python dict of metadata key value pairs (Optional)
        testMode -- true/false (default=False)

        Returns: status code
        """

        if testMode:
            self.baseURL('entities/'+str(entityId)+'/test-alarm') 
        else:
            self.baseURL('entities/'+str(entityId)+'/alarms') 
        self.attrDict = attrDict

        #required args
        self.postVar('check_id',checkId)
        self.postVar('notification_plan_id',planId)

        #optional args
        self.postDict()

        if testMode:
            return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        else:
            return apiRequest(self, self.endpoint, "POST", self.postData)
        
    def testAlarm(self, entityId, checkId, planId, attrDict={}):
        """
        Test run the creation of an alarm.
        
        Arguments:
        entityId --
        checkId --
        planId -- notification plan id
        attrDict -- python dict options:
            criteria -- (Optional)
            label -- (Optional)
            metadataDict -- python dict of metadata key value pairs (Optional)

        Returns: status code
        """
        return self.createAlarm(entityId, checkId, planId, attrDict, True)
        
    def getAlarm(self, entityId, alarmId):
        """
        Returns the specified alarm.
        
        Arguments:
        entityId -- 
        alarmId -- 
        
        Returns: status code.
        """
        self.baseURL('entities/'+str(entityId)+'/alarms/'+str(alarmId)) 
        return apiRequest(self, self.endpoint, "GET", self.postData, returnResults=True)
        
    def getAlarms(self, entityId):
        """
        Lists alarms associated with an entityId.
        
        Arguments:
        entityId --
        
        Returns: JSON results as python dict
        """
        self.baseURL('entities/'+str(entityId)+'/alarms') 
        return apiRequest(self, self.endpoint, "GET", self.postData, returnResults=True)
        
    def updateAlarm(self, entityId, alarmId, attrDict={}):
        """
        Update an alarm associated with an entity.
        
        Arguments:
        entityId --
        alarmId --
        attrDict -- python dict options:
            checkId --
            planId -- notification plan id (Optional)
            criteria -- (Optional)
            label -- (Optional)
            metadataDict -- python dict of metadata key value pairs (Optional)

        Returns: status code
        """

        self.baseURL('entities/'+str(entityId)+'/alarms/'+str(alarmId)) 
        self.attrDict = attrDict

        #optional args
        self.postDict()

        #return apiRequest(self, self.endpoint, "PUT", self.postData, returnResults=True)
        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteAlarm(self, entityId, alarmId):
        """
        Deletes an alarm.
        
        Arguments:
        entityId -- 
        alarmId -- 
        
        Returns: status code
        """
        self.baseURL('entities/'+str(entityId)+'/alarms/'+str(alarmId)) 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Notification Plans {{{

    def createNPlan(self, label, attrDict={}): # criticalState, okState, warningState):
        """
        Creates a notification plan.
        
        Arguments:
        label --
        attrDict -- python dict options:
            critical_state --
            ok_state --
            warning_state --

        Returns: status code
        """

        self.baseURL('notification_plans') 
        self.attrDict = attrDict

        #optional args
        self.postVar('label',label)
        self.postDict()

        #print self.endpoint
        #print self.postData

        #return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        return apiRequest(self, self.endpoint, "POST", self.postData)
        
    def getNPlans(self):
        """
        Lists notification plans for current account.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('notification_plans') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getNPlan(self, npId):
        """
        Returns the specified notification plan.
        
        Arguments:
        nplanId -- 
        
        Returns: status code.
        """
        self.baseURL('notification_plans/'+str(npId)) 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def updateNPlan(self, npId, attrDict={}):
        """
        Update a notification plan associated with an entity.
        
        Arguments:
        nplanId --
        attrDict -- python dict of option args:
            label --
            critical_state --
            ok_state --
            warning_state --

        Returns: status code
        """

        self.baseURL('notification_plans/'+str(npId)) 
        self.attrDict = attrDict
        self.postDict()
        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteNPlan(self, npId):
        """
        Deletes a notification plan.
        
        Arguments:
        entityId -- 
        nplanId -- 
        
        Returns: status code
        """
        self.baseURL('notification_plans/'+str(npId)) 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Monitoring Zones {{{

    def getMZones(self):
        """
        List monitoring zones.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('monitoring_zones') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getMZone(self, mzoneId):
        """
        Returns the specified monitoring zone.
        
        Arguments:
        mzoneId -- 
        
        Returns: status code.
        """
        self.baseURL('monitoring_zones/'+str(mzoneId)) 
        return apiRequest(self, self.endpoint, "GET", self.postData, returnResults=True)
        
    def traceMZone(self, mzoneId, target, attrDict={}):
        """
        Perform a traceroute from a collector in the specified monitoring zones.
        
        Arguments:
        mzoneId -- 
        target -- target ip or hostname
        attrDict -- python dict options:
            targetResolver -- IPv6 / IPv6 (Optional)
        
        Returns: status code.
        """
        self.baseURL('monitoring_zones/'+str(mzoneId)+'/traceroute') 
        self.attrDict = attrDict
        self.postVar('target',target)
        self.postDict()
        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)

    # }}}
    #Alarm Notification History {{{

    def discoverAlarmHistory(self, entityId, alarmId):
        """
        List checks for which alarm notification history is available.
        
        Arguments:
        entityId --
        alarmId --

        Returns: JSON results as python dict
        """
        self.baseURL('entities/'+str(entityId)+'/alarms/'+str(alarmId)+'/notification_history') 
        #print self.endpoint
        #print self.postData
        return apiRequest(self, self.endpoint, "GET", self.postData, returnResults=True)
        
    def getAlarmHistory(self, entityId, alarmId, checkId, uuid=None):
        """
        List checks for which alarm notification history is available -- or if
        uuid is provided, retrieves a single alarm notification history item.
        
        Arguments:
        entityId --
        alarmId --
        checkId --
        uuid -- id of a single alarm notification history item (Optional)

        Returns: JSON results as python dict
        """
        if uuid:
            self.baseURL('entities/'+str(entityId)+'/alarms/'+str(alarmId)+'/notification_history/'+str(checkId)+'/'+str(uuid)) 
        else:
            self.baseURL('entities/'+str(entityId)+'/alarms/'+str(alarmId)+'/notification_history/'+str(checkId)) 
        #print self.endpoint
        #print self.postData
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Notifications {{{

    def createNotification(self, label, type, details, testMode=False):
        """
        Create (or test the creation of ) a notification.
        
        Arguments:
        label -- 
        type -- webhook / email
        details -- 
        testMode -- true/false (default=False)
        
        Returns: status code
        """
        self.baseURL('notifications')
        self.postVar('details',details)
        self.postVar('label',label)
        self.postVar('type',type)
        return apiRequest(self, self.endpoint, "POST", self.postData)
        
    def testNotification(self, label, type, details):
        """
        Tests the creation of a notification.
        
        Arguments:
        label -- 
        type -- webhook / email
        details -- 
        
        Returns: status code
        """
        return self.createNotification(label,type,details,True)
        
    def getNotification(self, noteId):
        """
        Returns the specified notification.
        
        Arguments:
        noteId -- 
        
        Returns: status code.
        """
        self.baseURL('notifications/'+str(noteId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getNotifications(self):
        """
        Lists notifications.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('notifications')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def testRunNotification(self, noteId):
        """
        Test a notification inline.
        
        Arguments:
        noteId -- 
        
        Returns: status code.
        """
        self.baseURL('notifications/'+str(noteId)+'/test')
        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def updateNotification(self, noteId, attrDict={}): 
        """
        Updates a notification.
        
        Arguments:
        noteId --
        attrDict -- python dict options:
            details -- 
            label -- 
            type -- webhook / email
        
        Returns: status code
        """

        self.baseURL('notifications/'+str(noteId))
        self.attrDict = attrDict

        #optional args
        self.postDict()

        return apiRequest(self, self.endpoint, "PUT", self.postData)
        
    def deleteNotification(self, noteId):
        """
        Deletes a notification.
        
        Arguments:
        noteId -- 
        
        Returns: status code
        """
        self.baseURL('notifications/'+str(noteId))
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Notification Types {{{

    def getNotificationType(self, ntId):
        """
        Returns the specified notification type.
        
        Arguments:
        id -- notification type id
        
        Returns: status code.
        """
        self.baseURL('notification_types/'+str(ntId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getNotificationTypes(self):
        """
        List available notification types.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('notification_types')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Changelogs {{{

    def getAlarmChangelogs(self):
        """
        List alarm changelogs.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('changelogs/alarms')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Views {{{

    def getOverview(self):
        """
        Return the overview.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('views/overview')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Alarm Examples {{{

    def getAlarmExamples(self):
        """
        Return list of alarm examples.
        
        Arguments:
        
        Returns: JSON results as python dict
        """
        self.baseURL('alarm_examples')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getAlarmExample(self, aeId):
        """
        Retrieve alarm example.
        
        Arguments:
        aeId -- alarm example id
        
        Returns: status code.
        """
        self.baseURL('alarm_examples/'+str(aeId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def evalAlarmExample(self, aeId, evalDict):
        """
        Evaluate alarm example.
        
        Arguments:
        aeId -- alarm example id
        evalDict  -- python dict of key/value pairs

        Returns: status code.
        """
        self.baseURL('alarm_examples/'+str(aeId))
        self.postVar('values',evalDict)
        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)

    # }}}

#eof



#!/usr/bin/python

# Copyright 2012 Rackspace Hosting {{{
#
# Contact - Nate House nathan.house@rackspace.com
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# }}}

from Cloud import *

class CloudServersNG(Cloud):
    
    service = "ServersOpenStack"

    def getLimits(self):
        """
        Gets the current rate and absolute limits for your account.
        
        Returns:
        JSON results as python dict
        
        """
        endpoint = self.publicURL + "/limits"
        return apiRequest(self, endpoint, "GET", returnResults=True)
        
    def getExtensions(self, extension=None):
        """
        List available extensions and get details for a specific extension.
        
        Arguments:
        extension -- alias of extension(string)
        
        Returns:
        JSON results as python dict
        
        """
        
        endpoint = self.publicURL + "/extensions"
        
        if extension is not None:
            endpoint = endpoint + "/%s" % unicode(quote(str(extension)), 'utf-8')
            
        return apiRequest(self, endpoint, "GET", returnResults=True)

    #Servers{{{

    def getServer(self, sId):
        """
        Fetch a server's details.
        This operation returns the details of a specified server.
        
        Arguments:
        sId -- Server uuid
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('servers/'+str(sId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getServers(self, details=False, attrDict={}):
        """
        Fetch servers. Use details=True for all details. Use attrDict to filter results.
        
        Arguments:
        details -- bool, defaults to False
        attrDict Options:
            image -- The image ID (Optional)
            flavor -- The flavor ID (Optional)
            name -- The server name. (Optional)
            status -- The server status. (Optional)
            marker -- The ID of the last item in the previous list. (Optional)
            limit -- The page size. (Optional)
            changes-since --The changes-since time.
        """
        if details:
            self.baseURL('servers/detail') 
        else:
            self.baseURL('servers') 
        self.attrDict = attrDict
        self.queryDict()
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def createServer(self, name, flavorRef, imageRef, attrDict={}):
        """
        This operation asynchronously provisions a new server.
        
        Arguments:
        name --
        flavorRef --
        imageRef -- uuid(string)
        attrDict -- Optional dictionary containing OS-DCF:diskConfig, metadata, personality or accessIPv[4,6]
        
        Returns:
        JSON results as python dict
        """

        self.postKey='server'
        self.baseURL('servers')
        self.attrDict = attrDict

        #required args
        self.postKeyVar('name',name)
        self.postKeyVar('flavorRef',flavorRef)
        self.postKeyVar('imageRef',imageRef)

        #optional attrDict args
        self.postKeyDict()

        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def setServerAttr(self, sId, attrDict):
        """
        This operation updates the editable attributes of a specified server.
        
        Arguments:
        id -- Server uuid
        attrDict -- dictionary containing name, accessIPv[4,6] updates
        
        Returns:
        JSON results as python dict
        """

        self.postKey='server'
        self.baseURL('servers/'+str(sId))
        self.attrDict = attrDict

        #optional attrDict args (1 of which is required)
        self.postKeyDict()
        
        return apiRequest(self, self.endpoint, "PUT", self.postData, returnResults=True)
        
    def deleteServer(self, sId):
        """
        This operation deletes a specified server instance from the system.
        
        Arguments:
        sId -- Server uuid
        
        Returns:
        status code
        """
        self.baseURL('servers/'+str(sId))
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Server Addresses {{{

    def getAddresses(self, sId, network=None):
        """
        This operation lists all networks and addresses associated with a specified server.  Specify network for 
        filtering
        
        Arguments:
        sId -- Server uuid
        network -- Filter by network ie public, private
        
        Returns:
        JSON results as python dict
        
        """
        endpoint = self.publicURL + "/servers/%s/ips" % str(sId)
        if network is not None:
            endpoint = endpoint + "/%s" % str(network)
        return apiRequest(self, endpoint, "GET", returnResults=True)

    # }}}
    #Server Actions {{{    

    def _setAction(self, sId, actionList, returnResults=False):
        """
        Post requested action to api for included uuid.  
        
        Arguments:
        sId -- Server uuid
        actionList -- dictionary containing action request info
        """
        self.baseURL('servers/'+str(sId)+'/action')
        self.postData = actionList
        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults)
        
    def changePass(self, sId, adminPass):
        """
        This operation changes the administrator password for a specified server.
        
        Arguments:
        sId -- Server uuid
        adminPass -- new password
        
        Returns:
        status code
        """
        actionList = { 'changePassword': { 'adminPass': adminPass } }
        return self._setAction(sId, actionList)
        
    def rebootServer(self, sId, rType="SOFT"):
        """
        This operation performs a soft or hard reboot of a specified server. A soft reboot is a graceful 
        shutdown and restart of your server's operating system. A hard reboot power cycles your server, 
        which performs an immediate shutdown and restart.
        
        Arguments:
        sId -- Server uuid
        rType -- HARD or SOFT(string)
        
        Returns:
        status code
        """
        actionList = { 'reboot': { 'type': rType.upper() } }
        return self._setAction(sId, actionList)
        
    def rebuildServer(self, sId, name, flavorRef, imageRef, attrDict=None):
        """
        The rebuild operation removes all data on the server and replaces it with the specified image. The 
        serverRef and all IP addresses remain the same. If you specify name, metadata, accessIPv4, or accessIPv6 
        in the rebuild request, new values replace existing values. Otherwise, these values do not change.
        
        Arguments:
        sId -- Server uuid
        name -- Server name(string)
        flavorRef -- flavor number(string)
        imageRef -- uuid(string) 
        attrDict -- Optional dictionary containing OS-DCF:diskConfig, metadata, personality or accessIPv[4,6]
        
        Returns:
        JSON results as python dict
        """
        data = { 'rebuild': { 'name': str(name), 'flavorRef': str(flavorRef), 'imageRef': str(imageRef) } }
        if attrDict is not None:
            data = { 'servers': dict( data['servers'].items() + attrDict.items() ) }
        return self._setAction(sId, data, returnResults=True)
        
    def resizeServer(self, sId, flavorRef, diskConfig='AUTO'):
        """
        This operation converts an existing server to a different flavor, which scales the server up or down. The 
        original server is saved for a period of time to allow roll back if a problem occurs. You should test and 
        explicitly confirm all resizes. When you do so, the original server is removed. All resizes are automatically 
        confirmed after 24 hours if you do not explicitly confirm or revert the resize.
        
        Arguments:
        sId -- Server uuid
        flavorRef -- flavor id (string)
        diskConfig -- AUTO or MANUAL(string) OS-DCF:diskConfig
        
        Returns:
        status code
        """
        data = { 'resize': { 'flavorRef': str(flavorRef) } }
        if diskConfig.upper() == "MANUAL":
            data['resize']['OS-DCF:diskConfig'] = "MANUAL"
        return self._setAction(sId, data)
        
    def confirmResizeServer(self, sId):
        """
        After you verify that the newly resized server works properly, use this operation to confirm the resize. After 
        you confirm the resize, the original server is removed and you cannot roll back to that server. All resizes are 
        automatically confirmed after 24 hours if you do not explicitly confirm or revert the resize.
        
        Arguments:
        sId -- Server uuid
        
        Returns:
        status code
        """
        data = { 'confirmResize': None }
        return self._setAction(sId, data)
        
    def revertResizeServer(self, sId):
        """
        Use this operation to revert the resize and roll back to the original server. All resizes are automatically 
        confirmed after 24 hours if you do not explicitly confirm or revert the resize.
        
        Arguments:
        sId -- Server uuid
        
        Returns:
        status code
        """
        data = { 'revertResize': None }
        return self._setAction(sId, data)
        
    def rescueServer(self, sId):
        """
        A temporary root password is assigned for use during rescue mode. This password is returned in the response body 
        for this call.  Rescue mode is limited to 90 minutes, after which the rescue image is destroyed and the server 
        attempts to reboot. You can exit rescue mode at any time.
        
        Arguments:
        sId -- Server uuid
        
        Returns:
        JSON results as python dict
        """
        data = { 'rescue': None }
        
        return self._setAction(sId, data, returnResults=True)
        
    def unrescueServer(self, sId):
        """
        After you resolve any problems and reboot a rescued server, you can unrescue the server. When you unrescue the server, 
        the repaired image is restored to its running state with your original password.
        
        Arguments:
        sId -- Server uuid
        
        Returns:
        JSON results as python dict
        """
        data = { 'unrescue': None }
        return self._setAction(sId, data)
        
    def createImage(self, sId, name, metaDict=None):
        """
        This operation creates a new image for a specified server. Once complete, a new image is available that you can use to 
        rebuild or create servers.
        
        Arguments:
        sId -- Server uuid
        name -- Image name(string)
        metaDict -- dictionary containing metadata key/value pairs.(Optional)
        
        Returns:
        status code
        """
        data = {'createImage': {'name':str(name)}}
        if metaDict is not None:
            data['createImage']['metadata'] = metaDict
        return self._setAction(sId, data)

    # }}}
    #Volume Attachment Actions {{{

    def createAttVolume(self, sId, volumeList):
        """
        This operation attaches one or more volumes to the specified server.
        
        Arguments:
        sId -- Server uuid
        volumeList -- dictionary containing volumeId and device keys.
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('servers/'+str(sId)+'/os-volume-attachments')
        #endpoint = self.publicURL + "/servers/%s/os- volume_attachments" % str(sId)
        self.postVar('volumentAttachment',{volumeList})
        self.postData = { 'volumeAttachment': { volumeList } }
        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def getAttVolumes(self, sId, vId=None):
        """
        This operation returns a response body that lists the volume attachment(s) for the specified server.
        
        Arguments:
        sId -- Server uuid
        vId -- Volume attachment id(Optional)
        
        Returns:
        JSON results as python dict
        """
        #endpoint = self.publicURL + "/servers/%s/os-volume_attachments" % str(sId)
        if vId is not None:
            self.baseURL('servers/'+str(sId)+'/os-volume-attachments/'+str(vId))
            #endpoint = endpoint + "/%s" % str(vId)
        else:
            self.baseURL('servers/'+str(sId)+'/os-volume-attachments')
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def deleteAttVolume(self, sId, vId):
        """
        This operation deletes a specified volume attachment from a specified server instance.
        
        Arguments:
        sId -- Server uuid
        vId -- Volume attachment id
        
        Returns:
        status code
        """
        self.baseURL('servers/'+str(sId)+'/os-volume-attachments/'+str(vId))
        #endpoint = self.publicURL + "/servers/%s/os-volume_attachments/%s" % (str(sId), str(vId))
        return apiRequest(self, self.endpoint,"DELETE")

    # }}}
    #Flavors{{{

    def getFlavor(self, fId):
        """
        Fetch a flavor's details.
        
        Arguments:
        fId -- Flavor id
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('flavors/'+str(fId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getFlavors(self, details=False, attrDict=None):
        """
        Fetch flavors. Use details=True for all details. Use attrDict to filter results.
        
        Arguments:
        details -- Bool, defaults to False.  Specifying fId overrides this.
        attrDict -- Python dict containing any of the following:
            minDisk -- Minimum number of gigabytes of disk storage.
            minRam -- Minimum amount of RAM in megabytes.
            marker -- The ID of the last item in the previous list.
            limit -- Sets the page size.
        
        Returns:
        JSON results as python dict
        """
        if details:
            self.baseURL('flavors/details') 
        else:
            self.baseURL('flavors') 
        self.attrDict = attrDict
        self.queryDict()
        return apiRequest(self, self.endpoint, "GET", returnResults=True)

    # }}}
    #Images{{{

    def getImage(self, iId):
        """
        Fetch an image's details.
        
        Arguments:
        iId -- Image id
        
        Returns:
        JSON results as python dict
        """
        self.baseURL('images/'+str(iId))
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def getImages(self, details=False, attrDict=None):
        """
        Fetch images. Use details=True for all details. Use attrDict to filter results.
        
        Arguments:
        details -- Bool, defaults to False.  Specifying iId overrides this.
        attrDict -- Python dict containing any of the following:
            server -- Specify the server reference by ID or by full URL.
            name -- Filters the list of images by image name.
            status -- Filters the list of images by status.
            changes-since -- Filters the list of images to those that have changed since the changes-since time.
            marker -- The ID of the last item in the previous list.
            limit -- Sets the page size. 
            type -- {BASE|SERVER}
        
        Returns:
        JSON results as python dict
        """
        if details:
            self.baseURL('images/detail') 
        else:
            self.baseURL('images') 
        self.attrDict = attrDict
        self.queryDict()
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def deleteImage(self, iId):
        """
        This operation deletes the specified image from the system.
        
        Arguments:
        iId -- Image id
        
        Returns:
        status code
        """
        self.baseURL('images/'+str(iId))
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}
    #Metadata{{{

    def getMeta(self, mType, mId, key=None):
        """
        Lists metadata associated with a server/image or specified key.
        
        Arguments:
        mType -- servers / images
        mId -- Server or Image uuid
        key -- metadata key(optional)
        
        Returns:
        JSON response as python dict
        """
        if key is not None:
            self.baseURL(mType.lower()+'/'+str(mId)+'/metadata/'+str(key)) 
        else:
            self.baseURL(mType.lower()+'/'+str(mId)+'/metadata') 
        return apiRequest(self, self.endpoint, "GET", returnResults=True)
        
    def setMeta(self, mType, mId, metaDict, key=None):
        """
        Sets metadata for the specified server or image.  If no key is provided, this will replace all existing
        metadata with the provided metaDict key/value pairs.  If a key is provided, the list must contain that 
        key/value pair and only that one will get set.
        
        Arguments:
        mType -- servers / images
        mId -- Server or Image uuid
        metaDict -- Python dictionary containing key/value pairs to be added/replaced.
        key -- metadata key
        
        Returns:
        JSON results as python dict
        """

        if key is not None:
            self.baseURL(mType.lower()+'/'+str(mId)+'/metadata/'+str(key)) 
            self.postKey = 'meta'
            self.postKeyVar(str(key),metaDict[str(key)])
        else:
            self.baseURL(mType.lower()+'/'+str(mId)+'/metadata') 
            self.postVar('metadata',metaDict)

        return apiRequest(self, self.endpoint, "PUT", self.postData, returnResults=True)
        
    def updateMeta(self, mType, mId, metaList):
        """
        Updates metadata items for specified server or image.  Items that are not explicitly mentioned are not modified.
        
        Arguments:
        mType -- servers / images
        mId -- Server or Image uuid
        metaList -- Python dictionary containing key/value pairs to be udpated.
        
        Returns:
        JSON results as python dict
        """
        self.baseURL(mType.lower()+'/'+str(mId)+'/metadata') 
        self.postVar('metadata',metaList)
        return apiRequest(self, self.endpoint, "POST", self.postData, returnResults=True)
        
    def deleteMeta(self, mType, mId, key):
        """
        Deletes a metadata item.
        
        Arguments:
        mType -- servers / images
        mId -- Server or Image uuid
        key -- Metadata key to delete
        
        Returns:
        status code
        """
        self.baseURL(mType.lower()+'/'+str(mId)+'/metadata/'+str(key)) 
        return apiRequest(self, self.endpoint, "DELETE")

    # }}}

#eof



import distribute_setup
distribute_setup.use_setuptools()

from setuptools import setup, find_packages

setup(
    name = "RSCloud",
    version = "1.0.4",
    packages = ['rscloud','rscloud.bin'],
    scripts = ['distribute_setup.py'],
    install_requires = ['distribute>=0.6.28','requests>=1.1.0', 'pytz>=2012j', 'python-dateutil>=2.1','simplejson>=3.0.7','keyring>=1.2','PyYAML>=3.10'],
    setup_requires = ['distribute>=0.6.28'],
    author = "Rackspace SMB Operations",
    author_email = ["Nathan.House@RACKSPACE.com","Samuel.Stavinoha@RACKSPACE.com","Zak.Jones@RACKSPACE.com"],
    description = "Rackspace Cloud Python Bindings",
    license = "Apache License 2.0",
    keywords = ['rackspace', 'cloud', 'python bindings'],
    url = "http://pypi.python.org/pypi/RSCloud",
    include_package_data = True,
    classifiers=[
	  'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'Intended Audience :: Information Technology',
          'License :: OSI Approved :: Apache Software License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Software Development',
	         ],
    
     )
